! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
! Module: makethumbs_aux
!
! Authors:  Ian Stewart - SSC/LUX  (originally, makethumbs_mod.f90)
!           Masaaki Sakano - SSC/LUX 
!          
! Description: Functions for makethumb package.
!
! Comments: 
! 
! 2006/06 : XMMSRCNAME -> IAUNAME, RA -> RA_CORR, DEC -> DEC_CORR, 
! 2007/05/24 : RA/DEC is default (_NOT_ RA_CORR/DEC_CORR because the image frame is based on the uncorrected coordinates.)
! 
! TODO: 
!   Filename (SRC_NUM, digit, DET-ID?)
! 
!- 
module makethumbs_aux

  use cal
  use dal
  use errorHandling
!  use datafilter

  use dal_aux
  use wcs_aux
  use constants
  use coordinate
  use ssc_misc
  use array_utils
  use cal_aux

  use ingestsrcnames_aux

  implicit none 

  private 

  ! ------ Parameters (public) ------
  public LENOBSIDSTRPRINT
  public FMTOBSIDSTR
  public FMTDETIDSTR
  public LENINSTSTRPRINT
  public LENCOMMSTRPRINT
  public LENMAXTHUMBNAILFILENAME
  public MAXSTRINDEX

  ! ------ (a part of) Command line arguments (public) ------
  ! public ArgObsIdStr
  ! public ArgERangeStr
  ! public ArgInstStr
  ! public ArgCommLower
  ! public ArgCommUpper
  ! public ArgSizeIndexCross
  ! public ArgLWidth
  ! public ArgFontNumber
  ! public ArgColourMapId

  ! ------ Global variables (public) ------
  ! public ThumbnailHalfWidthPixels
  ! public ThumbnailHalfWidthArcmin
  ! public ImageXsize
  ! public ImageYsize

  ! ------ Functions/Subroutines ------
  public loadSources
  public getInstStr
  public calcSrcImageCoords
  public isSrcWithinFOV
  public makeAndSaveThumbnail
  public makeThumbnailGif
  public set_palette
  public writenumber
  public retDirname
  public isDirectoryReadable
  public isDirnameReadable
  public dumpThumbOne
  public dumpThumbCommon


  interface loadSources
    module procedure loadSourcesOneArgInterface
    module procedure loadSourcesOne, loadSourcesAry
  end interface

  interface calcSrcImageCoords
    module procedure calcSrcImageCoordsFromObj
    module procedure calcSrcImageCoordsFromSet, calcSrcImageCoordsFromName
  end interface

  interface makeAndSaveThumbnail
    module procedure makeAndSaveThumbnailFromObj ! , makeAndSaveThumbnailFromArgs
  end interface

  interface writenumber
    module procedure writeint8, writeint16, writeint32
    module procedure writereal, writedouble
  end interface


  interface fillSrcIndexForPrintStr
    ! Private function
    module procedure fillSrcIndexForPrintStrAry, fillSrcIndexForPrintStrOne
  end interface

  !.................
  ! Global parameters (public):

  integer, parameter :: LENOBSIDSTRPRINT = 10
  character(8), parameter :: FMTOBSIDSTR='(I10.10)'
  character(6), parameter :: FMTDETIDSTR='(I6.6)'
  integer, parameter :: LENINSTSTRPRINT = 16
  integer, parameter :: LENCOMMSTRPRINT = 255
  integer, parameter :: LENMAXTHUMBNAILFILENAME = 1023
  integer, parameter :: MAXSTRINDEX = 10

  !.................
  ! Type definition (public):

  type, public :: FilenameStyleT
    logical :: srcNum = .false. &
             , srcNumHex = .false. &
             , detId = .false. &
             , srcId = .false.
  end type FilenameStyleT

  type, public :: PgplotDisplayPrmT
    !! This is used in type(ThumbCommonT) and type(ArgTaskT) defined in MAIN.
    integer :: lWidth=0, fontNumber=0
    integer :: colourMapId=3	! 3 = heat
  end type PgplotDisplayPrmT

  type, public :: ThumbSizeInfoT
    !! This is used in type(ThumbCommonT)
    integer(int32) :: orgImageXSize, orgImageYSize	! Array size (=pixel)
    integer(int32) :: halfWidthPixels
    real(double)   :: halfWidthArcmin
    ! halfWidthArcmin = 20.0,&  !! Work fine
    ! halfWidthArcmin = 40.0,&	!! Doesn't work well!
    ! halfWidthArcmin = 8.0,&
    ! halfWidthArcmin = 4.0,&	!! Original
  end type ThumbSizeInfoT

  type, public :: ThumbOneT
    !! Parameters for each source for thumnail.
    !! You can view contents via  call dumpThumbOne()
    real(double) :: raDeg, decDeg
    integer(int32) :: srcIndex, detid, srcid
    real(single) :: xPixel, yPixel
    character(LENMAXTHUMBNAILFILENAME) :: filename = ''
    character(MAXSTRINDEX) :: indexStr = '', fullIndexStr = ''	! e.g., ":23456" when DETID==23456
    character(LENMAXTHUMBNAILFILENAME) :: catName = '', fullCatName = ''
    logical(bool) :: inCatFlag
  end type ThumbOneT

  type, public :: ThumbCommonT
    !! Common parameters in creating/processing thumnail GIFs.
    !! You can view contents via  call dumpThumbCommon()
    real(single), allocatable :: orgImage(:,:)
    integer(int32) :: obsId
    character(LENOBSIDSTRPRINT) :: obsIdStr=''
    character(LENINSTSTRPRINT)  :: eRangeStr='', instStr=''
    character(LENCOMMSTRPRINT)  :: commLower='', commUpper=''
    real(double) :: sizeIndexCross
    type(ThumbSizeInfoT) :: tSize	! Dimension etc of thumbnail image
    type(PgplotDisplayPrmT) :: prmPgplot
  end type ThumbCommonT

  ! type, public :: ThumbPlotParamT
  !   character(LENCOMMSTRPRINT)  :: srcName = '' &
  !      , fullSrcName   = 'DEFAULT' &
  !      , fullLowerLeft = 'DEFAULT'
  !      ! fullSrcName   <== trim(srcName)//trim(indexStr)  in default
  !      ! fullLowerLeft <== trim[obsIdStr, eRangeStr, instStr].join(" ")
  !   character(MAXSTRINDEX)      :: indexStr = ''	! eg., ":123456"
  !   character(LENOBSIDSTRPRINT) :: obsIdStr = ''
  !   character(LENINSTSTRPRINT)  :: eRangeStr = '', instStr = ''
  !   character(LENCOMMSTRPRINT)  :: commLower = '', commUpper = ''
  !   real :: sizeIndexCross = 0.0
  !   integer :: colourMapId = 3 &	! 3 = heat
  !      , lWidth = 0 &
  !      , fontNumber = 0
  ! end type ThumbPlotParamT

  !.................
  ! Global parameters (public/command-line-parameters):

  ! character(LENOBSIDSTRPRINT) :: ArgObsIdStr
  ! character(LENINSTSTRPRINT)  :: ArgERangeStr, ArgInstStr
  ! character(LENCOMMSTRPRINT)  :: ArgCommLower, ArgCommUpper
  ! real(double) :: &
  !    ArgSizeIndexCross
  ! integer :: &
  !    ArgLWidth, ArgFontNumber
  ! integer :: ArgColourMapId	! 3 = heat

  ! integer :: ThumbnailHalfWidthPixels
  ! real(double) :: ThumbnailHalfWidthArcmin
    !thumbnailHalfWidthArcmin = 20.0,&
    !thumbnailHalfWidthArcmin = 20.0,&  !! Work fine
    !thumbnailHalfWidthArcmin = 40.0,&	!! Doesn't work well!
    !thumbnailHalfWidthArcmin = 8.0,&
    !thumbnailHalfWidthArcmin = 4.0,&	!! Original
  ! integer :: &
  !   ImageXsize, ImageYsize

  !.................
  ! Global parameters (private):

  ! logical(bool) :: filenameWithSrcNum = .false., filenameWithSrcNumHex = .false.&
  !      , filenameWithDetId = .false., filenameWithSrcId = .false.
  ! ! This definition was moved from the main code, where they were
  ! ! used in subroutine loadSources_old() etc, while values are
  ! ! substituted in readParameters() in the main code.

contains 

  !---------------------------------------------------------------------
  subroutine loadSourcesOneArgInterface( &
      srcIndexStyle, fnameStyle, thumbOne, obsId &
    , withCoords, withCat, withSrcNum &
    , raDeg, decDeg, inCatSetTabName, srcNum)
  !---------------------------------------------------------------------
  ! Interface for loadSourcesOne() to work around present() constraints.
  ! See loadSourcesAry() for detail.

    implicit none 

    character(*), intent(in) :: srcIndexStyle
    type(FilenameStyleT), intent(in) :: fnameStyle
    type(ThumbOneT), intent(inout) :: thumbOne
    integer(int32), intent(in) :: obsId
    logical, intent(in) :: withCoords, withCat, withSrcNum
    real(double), intent(in), optional :: raDeg, decDeg
    character(*), intent(in), optional :: inCatSetTabName
    integer(int32), intent(in), optional :: srcNum	! ArgSrcIdNumber

    !-------------------------------------------------------------------
    ! Start:

    if (withCoords) then
      if (present(raDeg) .and. present(decDeg)) then
        ! OK
      else
        call error('inconsistentPrmLoadSources1', 'Contact the code developer.')
      end if
    end if

    if (withCat) then
      if (.not. present(inCatSetTabName)) then
        call error('inconsistentPrmLoadSources2', 'Contact the code developer.')
      end if
    end if

    if (withSrcNum) then
      if (.not. present(srcNum)) then
        call error('inconsistentPrmLoadSources3', 'Contact the code developer.')
      end if
    end if

    if (withCoords .and. withCat) then
      if (withSrcNum) then
        call loadSourcesOne( &
           srcIndexStyle, fnameStyle, thumbOne, obsId &
           , raDeg=raDeg, decDeg=decDeg &
           , inCatSetTabName=inCatSetTabName, srcNum=srcNum)
      else
        call loadSourcesOne( &
           srcIndexStyle, fnameStyle, thumbOne, obsId &
           , raDeg=raDeg, decDeg=decDeg &
           , inCatSetTabName=inCatSetTabName)
      end if

    else if (withCoords) then	! No catalogue given
      if (withSrcNum) then	! SRC_NUM for printing purpose only.
        call loadSourcesOne( &
           srcIndexStyle, fnameStyle, thumbOne, obsId &
           , raDeg=raDeg, decDeg=decDeg, srcNum=srcNum)
      else
        call loadSourcesOne( &
           srcIndexStyle, fnameStyle, thumbOne, obsId &
           , raDeg=raDeg, decDeg=decDeg)
      end if

    else if (withCat) then	! No coordinates given
      if (withSrcNum) then
        call loadSourcesOne( &
           srcIndexStyle, fnameStyle, thumbOne, obsId &
           , inCatSetTabName=inCatSetTabName, srcNum=srcNum)
      else
          call error('NoSrcNumSpecified' &
             , 'srcnum is mandatory when withsrclist=no and withcat=yes.')
          ! copied from readParameters() in makethumbs_mod.f90.
          ! Should be unnecessary, but just to play safe.
      end if
    else
        call error('inconsistentPrmLoadSources4', 'Contact the code developer.')
    end if

  end subroutine loadSourcesOneArgInterface


  !---------------------------------------------------------------------
  subroutine loadSourcesOne( &
      srcIndexStyle, fnameStyle, thumbOne, obsId &
    , raDeg, decDeg, inCatSetTabName, srcNum)
  !---------------------------------------------------------------------
  ! This is for just one source (and accordingly one image).
  ! See loadSourcesAry() for detail.

    implicit none 

    character(*), intent(in) :: srcIndexStyle
    type(FilenameStyleT), intent(in) :: fnameStyle
    type(ThumbOneT), intent(inout) :: thumbOne
    integer(int32), intent(in) :: obsId
    real(double), intent(in), optional :: raDeg, decDeg
    character(*), intent(in), optional :: inCatSetTabName
    integer(int32), intent(in), optional :: srcNum	! ArgSrcIdNumber

    type(SrcPrmT) :: srcPrm	! defined in ingestsrcnames_aux
    ! real(double), pointer :: srcRaDeg(:), srcDecDeg(:)
    ! integer(int32), pointer :: srcIndex(:), srcDetid(:), srcSrcid(:)
    ! character(LENMAXTHUMBNAILFILENAME), pointer :: srcCatName(:)
    ! logical(bool), pointer :: srcInCatFlag(:)
    integer, parameter :: numRows = 1
    integer(int32) :: srcNumNow

    ! character(LENSTRFILENAME) :: catSetName	! LEN... defined in ingestsrcnames_aux
    ! character(LENONEFITSLINE) :: catTabName	! LEN... defined in ingestsrcnames_aux
    ! character(LEN=80) :: headRa = 'RA', headDec = 'DEC'
    ! character(LEN=80) :: headDetid = 'DETID', headSrcid = 'SRCID'
    ! character(LEN=80) :: headSrcname = 'IAUNAME'
    ! character(LEN=80) :: headSrcnum = 'SRC_NUM', headIncatflg = ''
    logical :: readCat
    ! integer :: iSrc	!, iSrcBegin, iSrcEnd
    ! integer, parameter :: iSrcBegin = 1, iSrcEnd = 1

    !-------------------------------------------------------------------
    ! Start:

    if (present(srcNum)) then
      srcNumNow = srcNum
    else
      srcNumNow = 0	! Default
    end if

    !-- Allocate pointers related to the source list.
    call allocateSrcPrm(srcPrm, numRows &
       , srcNum=.true., detid=.true., srcid=.true. &
       , inCatFlag=.true., catName=.true., raDeg=.true., decDeg=.true.)	! defined in ingestsrcnames_aux
      !! Make sure to deallocate them later on!

    ! allocate(srcPrm%raDeg(numRows), srcPrm%decDeg(numRows)&
    !   , srcPrm%srcNum(numRows), srcPrm%detid(numRows), srcPrm%srcid(numRows) &
    !   , srcPrm%inCatFlag(numRows), srcPrm%catName(numRows))
    ! srcPrm%flagAlloc%srcNum    = .true.
    ! srcPrm%flagAlloc%detid     = .true.
    ! srcPrm%flagAlloc%srcid     = .true.
    ! srcPrm%flagAlloc%inCatFlag = .true.
    ! srcPrm%flagAlloc%catName   = .true.
    ! srcPrm%flagAlloc%raDeg     = .true.
    ! srcPrm%flagAlloc%decDeg    = .true.
    srcPrm%catName(:) = ''

    !! Check for the wrong use of this subroutine.
    if ( (      present(raDeg)) .and. (.not. present(decDeg))  .or. &
         (.not. present(raDeg)) .and. (      present(decDeg)) ) then
      call error('inConsistentRaDecCall', 'Contact the code developer.')
    end if
    if ( (.not. present(raDeg)) .and. (.not. present(decDeg)) .and. &
         (.not. present(inCatSetTabName)) ) then
      call error('noCoordsSpecified' &
         , 'Either the coordinates or catalogue and SRC_NUM must be specified.')
    end if

    if (present(raDeg))  srcPrm%raDeg  = raDeg 
    if (present(decDeg)) srcPrm%decDeg = decDeg

    if (present(inCatSetTabName)) then
      readCat = .true.
    else
      readCat = .false.
    end if
    if (present(srcNum)) then
      srcPrm%srcNum = srcNum
    else
      readCat = .false.
      if (present(inCatSetTabName)) then
        call warning('catalogueNotRead' &
           , 'SRC_NUM is not specified, hence the catalogue ('&
           //trim(inCatSetTabName)//') is not read.')
      end if
    end if

    !---- Read DETID, SRCID, IAUNAME, INCATFLG
    !-- Open 2XMM-etc. catalogue if specified.
    if (readCat) then
      call message(AppMsg, Noisy, 'Reading Catalogue ('//trim(inCatSetTabName)//').')
      call getCatBasedPrmsFromSetTabName(inCatSetTabName, srcPrm &
         , obsId=obsId, getRaDec=(.not. present(raDeg)))	! in ingestsrcnames_aux

    else	! if (present(inCatSetTabName)) then
      srcPrm%inCatFlag = .true.
    end if	! if (present(inCatSetTabName)) then


    ! Load parameters (pointer) to the output array (fixed size).
    thumbOne%raDeg     = srcPrm%raDeg(1)
    thumbOne%decDeg    = srcPrm%decDeg(1)
    thumbOne%srcIndex  = srcPrm%srcNum(1)
    thumbOne%detid     = srcPrm%detid(1)
    thumbOne%srcid     = srcPrm%srcid(1)
    thumbOne%catName   = trim(srcPrm%catName(1))
    thumbOne%inCatFlag = srcPrm%inCatFlag(1)
    ! thumbOnes(:)%indexStr  = srcIndexForPrintStr
    ! Now, %filename, %fullIndexStr, %fullCatName are not filled, yet.

    call fillSrcIndexForPrintStrOne(thumbOne, srcIndexStyle)

    if (verbosity(AppMsg) >= Noisy) then
call message(AppMsg, Noisy, 'The catalogue source parameters before loadig are ... ')
call dumpSrcPrm(srcPrm, 1)
      call message(AppMsg, Noisy, 'The thumbnail parameters after loading are ... ')
      call dumpThumbOne(thumbOne)
    end if

    call deallocateSrcPrm(srcPrm &
       , iSrc=1, varName='srcPrm', minVerbosity=Noisy)	! in ingestsrcnames_aux

  end subroutine loadSourcesOne


  !---------------------------------------------------------------------
  subroutine loadSourcesAry(sourceTab, inSrcListSetTabName &
    , srcIndexStyle, fnameStyle, thumbOnes &
    , obsId, inCatSetTabName, srcNum)
  !---------------------------------------------------------------------
  ! This is meant to read the EPIC summary source list.

!*** NOTE! the old version of loadSources filtered the source list itself but (in order to avoid conflicts over the name of the temp cloned file) this version doesn't clone or filter the file.

    implicit none 

    type(TableT), intent(in) :: sourceTab
    character(*), intent(in) :: inSrcListSetTabName, srcIndexStyle
    type(FilenameStyleT), intent(in) :: fnameStyle
    type(ThumbOneT), intent(inout) :: thumbOnes(:)
    integer(int32), intent(in) :: obsId
      ! NOTE: You may think it is possible to get OBS-ID from the header
      ! of the input source-list file.  However, because the first argument
      ! is TableT, it is not possible to get it from its primary header,
      ! where it is most likely OBS-ID is stated.
      ! Therefore, I have decided to force users of this library
      ! to give ObsId (Integer32) as the argument.
      ! Another note is that the File-handle passed to this subroutine
      ! as the argument has to be TableT, not DataSetT or filename;
      ! the reason is I would like thumbOnes(:)
      ! to be allocated before the call of this routine and then
      ! it is necessary anyway for the caller to access TableT
      ! before the call in order to know numberOfRows of the Table,
      ! which is the dimension of thumbOnes(:). 
    character(*), intent(in), optional :: inCatSetTabName
    integer(int32), intent(in), optional :: srcNum	! ArgSrcIdNumber

    type(SrcPrmT) :: srcPrm	! defined in ingestsrcnames_aux
    ! real(double), pointer :: srcRaDeg(:), srcDecDeg(:)
    ! integer(int32), pointer :: srcIndex(:), srcDetid(:), srcSrcid(:)
    ! character(LENMAXTHUMBNAILFILENAME), pointer :: srcCatName(:)
    ! logical(bool), pointer :: srcInCatFlag(:)
    integer :: numRows
    integer(int32) :: srcNumNow

    character(LEN=80) :: headRa = 'RA', headDec = 'DEC' &
                       , headDetid = 'DETID', headSrcid = 'SRCID' &
                       , headSrcname = 'IAUNAME' &
                       , headSrcnum = 'SRC_NUM', headIncatflg = ''
    logical :: useDetId, useSrcId, useCatName
    ! integer :: iSrc, iSrcBegin, iSrcEnd
    ! character(MAXSTRINDEX), allocatable :: srcIndexForPrintStr(:)

    !-------------------------------------------------------------------
    ! Start:

    if (present(srcNum)) then
      srcNumNow = srcNum
    else
      srcNumNow = 0	! Default
    end if

    !-- Allocate pointers related to the source list.
    numRows = numberOfRows(sourceTab)
    call allocateSrcPrm(srcPrm, numRows &
       , srcNum=.true., detid=.true., srcid=.true. &
       , inCatFlag=.true., catName=.true., raDeg=.true., decDeg=.true.)	! defined in ingestsrcnames_aux
      !! Make sure to deallocate them later on!

    ! allocate(srcPrm%raDeg(numRows), srcPrm%decDeg(numRows)&
    !   , srcPrm%srcNum(numRows), srcPrm%detid(numRows), srcPrm%srcid(numRows) &
    !   , srcPrm%inCatFlag(numRows), srcPrm%catName(numRows))
    ! srcPrm%flagAlloc%srcNum    = .true.
    ! srcPrm%flagAlloc%detid     = .true.
    ! srcPrm%flagAlloc%srcid     = .true.
    ! srcPrm%flagAlloc%inCatFlag = .true.
    ! srcPrm%flagAlloc%catName   = .true.
    ! srcPrm%flagAlloc%raDeg     = .true.
    ! srcPrm%flagAlloc%decDeg    = .true.
    ! 
    ! ! allocate(srcRaDeg(numRows), srcDecDeg(numRows)&
    ! !   , srcIndex(numRows), srcDetid(numRows), srcSrcid(numRows), srcCatName(numRows)&
    ! !   , srcInCatFlag(numRows))

    !! Check table column names
    headRa     = getRaStr(sourceTab)		! in ingestsrcnames_aux
    headDec    = getDecStr(sourceTab)		! in ingestsrcnames_aux
    headSrcnum = getSrcNumStr(sourceTab)	! in ingestsrcnames_aux

    call readColDataToFixed(sourceTab, trim(headRa),      srcPrm%raDeg) ! trim may not be needed.
    call readColDataToFixed(sourceTab, trim(headDec),     srcPrm%decDeg)
    call readColDataToFixed(sourceTab, trim(headSrcnum),  srcPrm%srcNum)

    !---- Read DETID, SRCID, IAUNAME, INCATFLG from either the source list or external catalogue.

    !---- Read DETID, SRCID, IAUNAME from the source list
    headDetid = getDetIdStr(  sourceTab, noErr=.true.)	! in ingestsrcnames_aux
    headSrcid = getSrcIdStr(  sourceTab, noErr=.true.)	! in ingestsrcnames_aux
    headSrcname=getIAUNameStr(sourceTab, noErr=.true.)		! in ingestsrcnames_aux

    !-- Open 2XMM-etc. catalogue if specified.
    if (present(inCatSetTabName)) then
      if (trim(headDetid)   .ne. '') then
        call warning('NotUseDetidInSrcList',  'The column ('//trim(headDetid) &
           //') is found in the source list ('//trim(inSrcListSetTabName) &
           //') but that from the Catalogue (' &
           //trim(inCatSetTabName)//') will be used instead.')
      end if
      if (trim(headSrcid)   .ne. '') then
        call warning('NotUseSrcidInSrcList',  'The column ('//trim(headSrcid) &
           //') is found in the source list ('//trim(inSrcListSetTabName) &
           //') but that from the Catalogue (' &
           //trim(inCatSetTabName)//') may be used instead, if found.')
      end if
      if (trim(headSrcname) .ne. '') then
        call warning('NotUseCatnameInSrcList','The column ('//trim(headSrcname) &
           //') is found in the source list ('//trim(inSrcListSetTabName) &
           //') but that from the Catalogue (' &
           //trim(inCatSetTabName)//') will be used instead.')
      end if

      call message(AppMsg, Noisy, 'Reading Catalogue ('//trim(inCatSetTabName)//').')
      call getCatBasedPrmsFromSetTabName(inCatSetTabName, srcPrm &
         , obsId=obsId, getRaDec=.false.)	! in ingestsrcnames_aux

      ! call getCatBasedPrmsFromSetTabName(inCatSetTabName, obsId &
      !    , srcPrm%srcNum, srcPrm%detid, srcPrm%srcid &
      !    , srcPrm%catName, srcPrm%inCatFlag)	! in ingestsrcnames_aux
      !    ! , srcRaDeg, srcDecDeg)

    else	! if (present(inCatSetTabName)) then

      useDetId = .true.
      if (trim(headDetid) == '') then
        if ( (trim(srcIndexStyle) .ne. 'detid') .and. (.not. fnameStyle%detId) ) then
          ! DETID is not used anyway.
          useDetId = .false.
        else
          call error('NotFoundDETID','DETID is not found in the table: '&
             //trim(inSrcListSetTabName))
        end if
      end if
!print *,'read0010=',trim(headDetid)
      if (useDetId) call readColDataToFixed(sourceTab, trim(headDetid), srcPrm%detid)

      useSrcId = .true.
      if (trim(headSrcid) == '') then
        if ( (trim(srcIndexStyle) .ne. 'srcid') .and. (.not. fnameStyle%srcId) ) then
          ! SRCID is not used anyway.
          useSrcId = .false.
        else
          call error('NotFoundSRCID','SRCID is not found in the table: '&
             //trim(inSrcListSetTabName))
        end if
      end if
      if (useSrcId) call readColDataToFixed(sourceTab, trim(headSrcid), srcPrm%srcid)
        ! Even if useSrcId is true, it does not always mean it is used afterward.

      if (trim(headSrcname) == '') then
        ! Do nothing.
        srcPrm%catName = ''
      else
        call readColDataToFixed(sourceTab, trim(headSrcname), srcPrm%catName)
      end if

      headIncatflg = getInCatFlgStr(sourceTab, noErr=.true.)	! in ingestsrcnames_aux
      if (trim(headIncatflg) == '') then	! INCATFLG not exists.
        srcPrm%inCatFlag = logical(.true., bool)
      else
        call readColDataToFixed(sourceTab, trim(headIncatflg), srcPrm%inCatFlag)
      end if

    end if	! if (present(inCatSetTabName)) then


    ! Load parameters (pointer) to the output array (fixed size).
    thumbOnes(:)%raDeg     = srcPrm%raDeg
    thumbOnes(:)%decDeg    = srcPrm%decDeg
    thumbOnes(:)%srcIndex  = srcPrm%srcNum
    thumbOnes(:)%detid     = srcPrm%detid
    thumbOnes(:)%srcid     = srcPrm%srcid
    thumbOnes(:)%catName   = srcPrm%catName
    thumbOnes(:)%inCatFlag = srcPrm%inCatFlag
    ! thumbOnes(:)%indexStr  = srcIndexForPrintStr
    ! Now, %filename, %fullIndexStr, %fullCatName are not filled, yet.

    ! allocate(srcIndexForPrintStr(iSrcBegin:iSrcEnd))
    if ( srcNumNow <= 0 ) then
      call fillSrcIndexForPrintStrAry(thumbOnes, srcIndexStyle)
      ! iSrcBegin = lbound(thumbOnes, 1)
      ! iSrcEnd   = ubound(thumbOnes, 1)
    else
      call fillSrcIndexForPrintStrAry(thumbOnes, srcIndexStyle &
         , srcNumNow, srcNumNow)
      ! iSrcBegin = srcNumNow
      ! iSrcEnd   = srcNumNow
    end if


    call deallocateSrcPrm(srcPrm)	! in ingestsrcnames_aux
    ! deallocate(srcRaDeg, srcDecDeg, srcIndex, srcDetid, srcSrcid &
    !    , srcCatName, srcInCatFlag)

  end subroutine loadSourcesAry


  !---------------------------------------------------------------------
  subroutine fillSrcIndexForPrintStrAry(thumbOnes, srcIndexStyle &
     , srcBegin, srcEnd)
  !---------------------------------------------------------------------
  ! Fill the parameter of the string to print in the output image.

    implicit none 

    type(ThumbOneT), intent(inout) :: thumbOnes(:)
    character(*), intent(in) :: srcIndexStyle
    integer, intent(in), optional :: srcBegin, srcEnd

    integer :: iSrc, iSrcBegin, iSrcEnd

    !-------------------------------------------------------------------
    ! Start:

    iSrcBegin = lbound(thumbOnes, 1)
    iSrcEnd   = ubound(thumbOnes, 1)
    if (present(srcBegin)) iSrcBegin = srcBegin
    if (present(srcEnd))   iSrcEnd   = srcEnd  

    !! Make the array of SRC-IDs for printing (in the GIFs)
    select case(trim(srcIndexStyle))
    case('none')
      thumbOnes(:)%indexStr = ''
    case('srcnum')
      do iSrc = iSrcBegin, iSrcEnd
        call writenumber(thumbOnes(iSrc)%srcIndex &	! SRC_NUM
             , var=thumbOnes(iSrc)%indexStr) 
      end do
    ! case('hexsrcnum')
    case('detid')
      do iSrc = iSrcBegin, iSrcEnd
print *,'DETID =',thumbOnes(iSrc)%detid,' for ID=',iSrc
        call writenumber(thumbOnes(iSrc)%detid, fmt=FMTDETIDSTR &
             , var=thumbOnes(iSrc)%indexStr) 
      end do
    case('srcid')
      do iSrc = iSrcBegin, iSrcEnd
        call writenumber(thumbOnes(iSrc)%srcid &
             , var=thumbOnes(iSrc)%indexStr) 
      end do
    case default
      ! Do nothing
    end select

  end subroutine fillSrcIndexForPrintStrAry


  !---------------------------------------------------------------------
  subroutine fillSrcIndexForPrintStrOne(thumbOne, srcIndexStyle)
  !---------------------------------------------------------------------
  ! See fillSrcIndexForPrintStrAry() for detail.
  ! This one accepts a scalar instead of array.

    implicit none 

    type(ThumbOneT), intent(inout) :: thumbOne
    character(*), intent(in) :: srcIndexStyle

    type(ThumbOneT) :: thumbOnes(1)

    !-------------------------------------------------------------------
    ! Start:

    thumbOnes(:) = thumbOne
    call fillSrcIndexForPrintStrAry(thumbOnes, srcIndexStyle)
    thumbOne = thumbOnes(1)

  end subroutine fillSrcIndexForPrintStrOne


!!! NOTE:
!! This subroutine code used to be in the main code, makethumb(s)_mod.f90
!! and it uses some global variables only valid in that file
!! (and that is why it is included in the main code).  Therefore
!! the code does not work any more as it is and would cause an error
!! in compiling; that is why it is commented out.  Nevertheless
!! it is left here for record.
!! If you think the code in this file has become matured enough,
!! feel free to delete the following piece of code.
!!
!
!   !---------------------------------------------------------------------
!   subroutine loadSources_old(inSrcListSetTabName, srcRaDeg, srcDecDeg, srcIndex &
!     , srcDetid, srcSrcid, srcCatName, srcInCatFlag, inCatSetTabName, obsId)
!   !---------------------------------------------------------------------
!   ! This is meant to read the EPIC summary source list.
!   ! Make sure to deallocate(srcRaDeg, srcDecDeg, srcIndex, srcDetid, srcCatName, srcInCatFlag) after use.
! 
! !*** NOTE! the old version of loadSources filtered the source list itself but (in order to avoid conflicts over the name of the temp cloned file) this version doesn't clone or filter the file.
! 
!     implicit none 
! 
!     character(*), intent(in) :: inSrcListSetTabName
!     real(double), pointer ::&
!       srcRaDeg(:), srcDecDeg(:)
!     integer(int32), pointer :: srcIndex(:), srcDetid(:), srcSrcid(:)
!     character(*), pointer :: srcCatName(:)
!     logical(bool), pointer :: srcInCatFlag(:)
!     character(*), intent(in), optional :: inCatSetTabName
!     integer(int32), intent(in), optional :: obsId	! maybe obtained with getObsIdInt32FromHeaderFromSet(sumSet)
! 
!     integer(int32) :: obsIdNow
!     integer :: numRows
!     type(DataSetT) :: sourceSet, catSet
!     type(TableT) :: sourceTab
! 
!     character(LENSTRFILENAME) :: srcListSetName, catSetName	! LEN... defined in ingestsrcnames_aux
!     character(LENONEFITSLINE) :: srcListTabName, catTabName	! LEN... defined in ingestsrcnames_aux
!     character(LEN=80) :: headRa = 'RA', headDec = 'DEC'
!     character(LEN=80) :: headDetid = 'DETID', headSrcid = 'SRCID'
!     character(LEN=80) :: headSrcname = 'IAUNAME'
!     character(LEN=80) :: headSrcnum = 'SRC_NUM', headIncatflg = ''
!     logical :: noColonFoundCat, noColonFoundSrcList
!     logical :: useDetId = .true., useSrcId = .true.
! 
!     !-------------------------------------------------------------------
!     ! Start:
! 
!     !-- Open the source list.
!     call splitSetTabName(inSrcListSetTabName, srcListSetName, srcListTabName &
!        , noColonFoundSrcList)	! in ssclib/dal_aux
!     !call splitSetTabName(ArgSrcListSetTabName, srcListSetName, srcListTabName)
! 
!     sourceSet = dataSet(srcListSetName, READ)
!     sourceTab = table(sourceSet, srcListTabName, READ)
!     if (verbosity(AppMsg) >= Noisy) then
!       call message(AppMsg, Noisy, 'SRC-List name is '//trim(srcListSetName))
!       if (.not. noColonFoundSrcList) then
!         call message(AppMsg, Noisy, 'SRC-List Table name is '//trim(srcListTabName))
!       else
!         call message(AppMsg, Noisy, 'No SRC-List Table name is given (Def: '&
!            //trim(DEFAULTSRCTBLNAME)//').')
!       end if
!     end if
! 
!     !-- Allocate pointers related to the source list.
!     numRows = numberOfRows(sourceTab)
!     allocate(srcRaDeg(numRows), srcDecDeg(numRows)&
!       , srcIndex(numRows), srcDetid(numRows), srcSrcid(numRows), srcCatName(numRows)&
!       , srcInCatFlag(numRows))
!       !! Make sure to deallocate them later on!
! 
!     !! Check table column names
!     headRa     = getRaStr(sourceTab)		! in ingestsrcnames_aux
!     headDec    = getDecStr(sourceTab)		! in ingestsrcnames_aux
!     headSrcnum = getSrcNumStr(sourceTab)	! in ingestsrcnames_aux
! 
!     call readColDataToFixed(sourceTab, trim(headRa),      srcRaDeg) ! trim may not be needed.
!     call readColDataToFixed(sourceTab, trim(headDec),     srcDecDeg)
!     call readColDataToFixed(sourceTab, trim(headSrcnum),  srcIndex)
! 
!     ! if (hasColumn(sourceTab, 'RA')) then
!     !   headRa = 'RA'	! Default
!     ! else if (hasColumn(sourceTab, 'ra')) then
!     !   headRa = 'ra'
!     ! else if (hasColumn(sourceTab, 'RA_UNC')) then
!     !   headRa = 'RA_UNC'
!     ! else if (hasColumn(sourceTab, 'ra_unc')) then
!     !   headRa = 'ra_unc'
!     ! else if (hasColumn(sourceTab, 'RA_CORR')) then
!     !   headRa = 'RA_CORR'
!     ! else
!     !   headRa = 'ra_corr'
!     ! end if
!     ! 
!     ! if (hasColumn(sourceTab, 'DEC')) then
!     !   headDec = 'DEC'	! Default
!     ! else if (hasColumn(sourceTab, 'dec')) then
!     !   headDec = 'dec'
!     ! else if (hasColumn(sourceTab, 'DEC_UNC')) then
!     !   headDec = 'DEC_UNC'
!     ! else if (hasColumn(sourceTab, 'dec_unc')) then
!     !   headDec = 'dec_unc'
!     ! else if (hasColumn(sourceTab, 'DEC_CORR')) then
!     !   headDec = 'DEC_CORR'
!     ! else
!     !   headDec = 'dec_corr'
!     ! end if
!     ! 
!     ! if (hasColumn(sourceTab, headSrcnum)) then
!     !   ! Default
!     ! else if (hasColumn(sourceTab, 'src_num')) then
!     !   headSrcnum = 'src_num'
!     ! else if (hasColumn(sourceTab, 'SRCNUM')) then
!     !   headSrcnum = 'SRCNUM'
!     ! else
!     !   headSrcnum = 'srcnum'
!     ! end if
! 
!     !---- Read DETID, SRCID, IAUNAME, INCATFLG
!     !-- Open 2XMM-etc. catalogue if specified.
!     if (present(inCatSetTabName)) then
! 
!       call splitSetTabName(inCatSetTabName, catSetName, catTabName &
!          , noColonFoundCat)	! in ssclib/dal_aux
!       if (verbosity(AppMsg) >= Noisy) then
!         call message(AppMsg, Noisy, 'Cataglogue name is '//trim(catSetName))
!         if (.not. noColonFoundCat) then
!           call message(AppMsg, Noisy, 'Cataglogue Table name is '//trim(catTabName))
!         else
!           call message(AppMsg, Noisy, 'No cataglogue Table name is given.')
!         end if
!       end if
! 
!       catSet = dataSet(catSetName, READ)
! 
!       !-- Get OBS-ID (Integer)
!       if (present(obsId)) then
!          obsIdNow = obsId
!       else
!          obsIdNow = getObsIdInt32FromHeader(sourceSet)	! in ingestsrcnames_aux
!       end if
! 
!       if (.not. noColonFoundCat) then
!         call getNew3Cols(catSet &
!            , srcCatName, srcIndex, srcDetid, srcSrcid, srcInCatFlag &
!            , obsIdNow, catTabName=catTabName)	! in ingestsrcnames_aux
!       else
!         call getNew3Cols(catSet &
!            , srcCatName, srcIndex, srcDetid, srcSrcid, srcInCatFlag &
!            , obsIdNow)				! in ingestsrcnames_aux
!       end if
! 
!       call release(catSet)
! 
!     else	! if (present(inCatSetTabName)) then
! 
!       headDetid = getDetIdStr(sourceTab, noErr=.true.)	! in ingestsrcnames_aux
!       if (trim(headDetid) == '') then
!         if ( (trim(ArgSrcIndexStyle) .ne. 'detid') .and. (.not. filenameWithDetId) ) then
!           ! DETID is not used anyway.
!           useDetId = .false.
!         else
!           call error('NotFoundDETID','DETID is not found in the table: '&
!              //trim(inSrcListSetTabName))
!         end if
!       end if
! 
!       headSrcid = getSrcIdStr(sourceTab, noErr=.true.)	! in ingestsrcnames_aux
!       if (trim(headSrcid) == '') then
!         if ( (trim(ArgSrcIndexStyle) .ne. 'srcid') .and. (.not. filenameWithDetId) ) then
!           ! SRCID is not used anyway.
!           useSrcId = .false.
!         else
!           call error('NotFoundSRCID','SRCID is not found in the table: '&
!              //trim(inSrcListSetTabName))
!         end if
!       end if
! 
!       headSrcname  = getIAUNameStr(sourceTab)		! in ingestsrcnames_aux
!       headIncatflg = getInCatFlgStr(sourceTab, noErr=.true.)	! in ingestsrcnames_aux
! 
! !print *,'read0010=',trim(headDetid)
!     if (useDetId) call readColDataToFixed(sourceTab, trim(headDetid), srcDetid)
! !print *,'read0011'
!     if (useSrcId) call readColDataToFixed(sourceTab, trim(headSrcid), srcSrcid)
!       ! Even if useSrcId is true, it does not always mean it is used afterward.
!     call readColDataToFixed(sourceTab, trim(headSrcname), srcCatName)
! 
!     if (trim(headIncatflg) == '') then	! INCATFLG not exists.
!       srcInCatFlag = logical(.true., bool)
!     else
!       call readColDataToFixed(sourceTab, trim(headIncatflg), srcInCatFlag)
!     end if
! 
!       ! if (hasColumn(sourceTab, headDetid)) then
!       !   ! Default
!       ! else if (hasColumn(sourceTab, 'detid')) then
!       !   headDetid = 'detid'
!       ! else if (hasColumn(sourceTab, 'DET_ID')) then
!       !   headDetid = 'DET_ID'
!       ! else if (hasColumn(sourceTab, 'det_id')) then
!       !   headDetid = 'det_id'
!       ! else	! DETID is not found.
!       !   if ((.not. printDetId) .and. (.not. filenameWithDetId)) then
!       !     ! DETID is not used anyway.
!       !     useDetId = .false.
!       !   else
!       !     call error('notFoundDetId','DETID is not found in the table.')
!       !   end if
!       ! end if
! 
!       ! if (hasColumn(sourceTab, headSrcname)) then
!       !   ! Default
!       ! else if (hasColumn(sourceTab, 'iauname')) then
!       !   headSrcname = 'iauname'
!       ! else
!       !   headSrcname = 'XMMSRCNAME'
!       ! end if
!       ! 
!       ! if (hasColumn(sourceTab, 'INCATFLG')) then
!       !   headIncatflg = 'INCATFLG'
!       ! else if (hasColumn(sourceTab, 'incatflg')) then
!       !   headIncatflg = 'incatflg'
!       ! else
!       !   headIncatflg = ''
!       ! end if
!     end if	! if (present(inCatSetTabName)) then
! 
!     call release(sourceSet)
! 
!   end subroutine loadSources_old


  !---------------------------------------------------------------------
  function getInstStr(inSet) result(instStr)
  !---------------------------------------------------------------------
  ! Get the string of instrument expression.

    implicit none 

    character(LENINSTSTRPRINT) :: instStr
    type(DataSetT), intent(in) :: inSet

    integer(int32) :: instrumentId
    character(LENINSTSTRPRINT) :: instrumentName	! for reading the FITS header
    real(double) :: expStartTimeStamp	! Dummy
    type(SpacecraftAttitudeType) :: scAttitude, instrumAttitude	! Dummy

    !-------------------------------------------------------------------
    ! Start:

    instrumentName = stringAttribute(inSet, "INSTRUME")
    if (trim(instrumentName) == 'EPIC') then
      instStr = 'EP'
    else 
      !call getCalInfo(inSetName, instrumentId)
      call getCalInfo(inSet, instrumentId, expStartTimeStamp &
         , scAttitude, instrumAttitude)	! in ssclib/cal_aux
      select case(instrumentId)
      case(EMOS1)
        ! instStr = 'emos1'
        instStr = 'M1'
      case(EMOS2)
        ! instStr = 'emos2'
        instStr = 'M2'
      case(EPN)
        ! instStr = 'epn'
        instStr = 'PN'
      !case(EPIC)		! Not defined in caloalutils/src/caloaldefs.f90
      !  instStr = 'EP'
      case default
        call release(inSet)
	call error('badInstrument','')	! <===================
      end select
    end if	! if (trim(instrumentName) == 'EPIC') then ... else ...

  end function getInstStr


  !---------------------------------------------------------------------
  subroutine calcSrcImageCoordsFromObj(inImageSet, thumbOnes, thumbCommon)
  !---------------------------------------------------------------------

    implicit none 

    type(DataSetT),     intent(in)    :: inImageSet
    type(ThumbOneT),    intent(inout) :: thumbOnes(:)
    type(ThumbCommonT), intent(inout) :: thumbCommon

    !-------------------------------------------------------------------
    ! Start:

    call calcSrcImageCoordsFromSet(inImageSet &
       , thumbOnes(:)%raDeg,  thumbOnes(:)%decDeg &
       , thumbOnes(:)%xPixel, thumbOnes(:)%yPixel &
       , thumbCommon%tSize)

  end subroutine calcSrcImageCoordsFromObj


  !---------------------------------------------------------------------
  subroutine calcSrcImageCoordsFromSet(inSet, srcRaDeg, srcDecDeg &
     , srcXpixels, srcYpixels, thumbSize)
  !---------------------------------------------------------------------

    implicit none 

    type(DataSetT), intent(in) :: inSet
    real(double), intent(in) ::&
      srcRaDeg(:), srcDecDeg(size(srcRaDeg))
    real(single), intent(out) ::&
      srcXpixels(size(srcRaDeg)), srcYpixels(size(srcRaDeg))
    type(ThumbSizeInfoT), intent(inout) :: thumbSize

    type(ArrayT) :: tmpArray
    type(WcsT) :: imageWcs
    integer :: numSources
    real(double), allocatable ::&
      srcTanX(:),&
      srcTanY(:)
real(double) :: xPixelSizeWorld, yPixelSizeWorld, rotAngle
    real(double) :: arcsecPerPixelX

    !-------------------------------------------------------------------
    ! Start:

    ! call readWcs(inSet, imageWcs)
      ! The above may not work because it is unknown where WCS keywords exist,
      ! whether the primary or the 1st extention header.
      ! Therefore here it will call readWcsSetArray() in ssclib/wcs_aux
      ! using also an ArrayT object.
      ! Note that it must be possible to check it with hasWcs() alternatively.
    tmpArray = array(inSet, 0, READ)
    call readWcs(inSet, tmpArray, imageWcs)	! in ssclib/wcs_aux.
    call release(tmpArray)
    if (verbosity(AppMsg) >= Noisy) then
      call dumpWcs(imageWcs, verbosity(AppMsg))	! in ssclib/wcs_aux.
    end if

call convertNewToOld(imageWcs, xPixelSizeWorld, yPixelSizeWorld, rotAngle) ! in ssclib/wcs_aux.
arcsecPerPixelX = abs(xPixelSizeWorld) * 3600.0d0
!    if (present(thumbSize)) then
      thumbSize%halfWidthPixels = thumbSize%halfWidthArcmin * 60.0d0 / arcsecPerPixelX
!ThumbnailHalfWidthPixels = ThumbnailHalfWidthArcmin * 60.0d0 / arcsecPerPixelX
      ! assumes pixels are square.
!    else
!ThumbnailHalfWidthPixels = ThumbnailHalfWidthArcmin * 60.0d0 / arcsecPerPixelX
!      ! assumes pixels are square.
!    end if

    numSources = size(srcRaDeg)
    allocate(srcTanX(numSources), srcTanY(numSources))
call raDecToTan(srcRaDeg*PiOn180, srcDecDeg*PiOn180&
  , imageWcs%x%refWorld*PiOn180, imageWcs%y%refWorld*PiOn180&
  , srcTanX, srcTanY)		! in ssclib/coordinate.

srcXpixels = (srcTanX / PiOn180) / abs(xPixelSizeWorld)&
  + imageWcs%x%refPixels
srcYpixels = (srcTanY / PiOn180) / abs(yPixelSizeWorld)&
  + imageWcs%y%refPixels

  end subroutine calcSrcImageCoordsFromSet


  !---------------------------------------------------------------------
  subroutine calcSrcImageCoordsFromName(inImageSetName, srcRaDeg, srcDecDeg &
     , srcXpixels, srcYpixels, thumbSize)
  !---------------------------------------------------------------------

    implicit none 

    character(*), intent(in) :: inImageSetName
    real(double), intent(in) ::&
      srcRaDeg(:), srcDecDeg(size(srcRaDeg))
    real(single), intent(out) ::&
      srcXpixels(size(srcRaDeg)), srcYpixels(size(srcRaDeg))
    type(ThumbSizeInfoT), intent(inout) :: thumbSize

    type(DataSetT) :: imageSet
!     type(WcsT) ::&
!       imageWcs
!     integer :: numSources
!     real(double), allocatable ::&
!       srcTanX(:),&
!       srcTanY(:)
! !    real(single) ::&
! !      xArcminPerPixel
! real(double) :: xPixelSizeWorld, yPixelSizeWorld, rotAngle
!     real(double) :: arcsecPerPixelX

    !-------------------------------------------------------------------
    ! Start:

    imageSet = dataSet(inImageSetName, READ)
    call calcSrcImageCoordsFromSet(imageSet, srcRaDeg, srcDecDeg &
       , srcXpixels, srcYpixels, thumbSize)
    call release(imageSet)

!     return
! 
! !    call getWcsInfo(imageSetName, imageWcs)
!     call readWcs(inImageSetName, imageWcs)	! in ssclib/wcs_aux.
!     call dumpWcs(imageWcs, verbosity(AppMsg))	! in ssclib/wcs_aux.
! 
! !    arcsecPerPixelX = abs(imageWcs%x%pixelSizeWorld) * 3600.0
! call convertNewToOld(imageWcs, xPixelSizeWorld, yPixelSizeWorld, rotAngle) ! in ssclib/wcs_aux.
! arcsecPerPixelX = abs(xPixelSizeWorld) * 3600.0d0
!     ThumbnailHalfWidthPixels = thumbnailHalfWidthArcmin * 60.0d0 / arcsecPerPixelX
!     ! assumes pixels are square.
! 
!     numSources = size(srcRaDeg)
!     allocate(srcTanX(numSources), srcTanY(numSources))
! !    call raDecToTan(imageWcs%x%refWorld, imageWcs%y%refWorld&
! !      , srcRaDeg, srcDecDeg, srcTanX, srcTanY)
! call raDecToTan(srcRaDeg*PiOn180, srcDecDeg*PiOn180&
!   , imageWcs%x%refWorld*PiOn180, imageWcs%y%refWorld*PiOn180&
!   , srcTanX, srcTanY)
! 
! !    srcXpixels = (srcTanX / PiOn180) / abs(imageWcs%x%pixelSizeWorld)&
! !      + imageWcs%x%refPixels
! !    srcYpixels = (srcTanY / PiOn180) / abs(imageWcs%y%pixelSizeWorld)&
! !      + imageWcs%y%refPixels
! srcXpixels = (srcTanX / PiOn180) / abs(xPixelSizeWorld)&
!   + imageWcs%x%refPixels
! srcYpixels = (srcTanY / PiOn180) / abs(yPixelSizeWorld)&
!   + imageWcs%y%refPixels

  end subroutine calcSrcImageCoordsFromName


  !---------------------------------------------------------------------
  function isSrcWithinFOV(thumbThis, thumbCommon) result(isTrue)
  !---------------------------------------------------------------------
  ! Return .true. if the source is within the FOV, else .false.

    implicit none 

    logical :: isTrue
    type(ThumbOneT),    intent(in) :: thumbThis
    type(ThumbCommonT), intent(in) :: thumbCommon

    real(single) :: x, y

    !-------------------------------------------------------------------
    ! Start:

    x = thumbThis%xPixel
    y = thumbThis%yPixel

    if (    (x < 1) .or. (x > thumbCommon%tSize%orgImageXSize) &
       .or. (y < 1) .or. (y > thumbCommon%tSize%orgImageYSize) ) then
      call warning('srcOutOfField', 'The source (' &
         //trim(thumbThis%fullCatName) &
         //') is out of the image field.  GIF file (' &
         //trim(thumbThis%filename)//') is not created.' )
      isTrue = .false.
    else
      isTrue = .true.
    end if

  end function isSrcWithinFOV


  !---------------------------------------------------------------------
  subroutine makeAndSaveThumbnailFromObj(thumbThis, thumbCommon)
  !---------------------------------------------------------------------
  ! Create the image array for the thumbnail-image area,
  ! then  call makeThumbnailGif().
  !
  ! NOTE: isSrcWithinFOV() should be called before calling this routine,
  !  and judge whether this routine can be called or not, because
  !  this routine just assumes that the source is within the FOV.

    implicit none 

    type(ThumbOneT),    intent(in) :: thumbThis
    type(ThumbCommonT), intent(in) :: thumbCommon

    real(single), allocatable :: thumbImage(:,:)
    real(single) :: srcXpixel, srcYpixel
    integer :: xSOrg, ySOrg, startXi, finisXi, startYi, finisYi
    type(ThumbSizeInfoT) :: tSize

    !-------------------------------------------------------------------
    ! Start:

    !! Making temporary variables (they are actually mere aliases).
    srcXpixel = thumbThis%xPixel
    srcYpixel = thumbThis%yPixel
    tSize = thumbCommon%tSize
    xSOrg = thumbCommon%tSize%orgImageXSize
    ySOrg = thumbCommon%tSize%orgImageYSize

    allocate( &
       thumbImage(-tSize%halfWidthPixels:tSize%halfWidthPixels &
                , -tSize%halfWidthPixels:tSize%halfWidthPixels) )

    thumbImage = 0.0
    startXi = max(1,     nint(srcXpixel) - tSize%halfWidthPixels)
    finisXi = min(xSOrg, nint(srcXpixel) + tSize%halfWidthPixels)
    startYi = max(1,     nint(srcYpixel) - tSize%halfWidthPixels)
    finisYi = min(ySOrg, nint(srcYpixel) + tSize%halfWidthPixels)

    thumbImage(startXi-nint(srcXpixel):finisXi-nint(srcXpixel)&
             , startYi-nint(srcYpixel):finisYi-nint(srcYpixel))&
       = thumbCommon%orgImage(startXi:finisXi, startYi:finisYi)

    call makeThumbnailGif(thumbImage, thumbThis, thumbCommon)

    ! call makeAndSaveThumbnailFromArgs(thumbThis%xPixel, thumbThis%yPixel &
    !      , thumbCommon%orgImage &
    !      , thumbThis%filename, thumbCommon%instStr &
    !      , thumbThis%fullIndexStr, thumbThis%fullCatName &
    !      , thumbCommon, thumbThis)
    !      ! , thumbCommon%tSize)

  end subroutine makeAndSaveThumbnailFromObj


!   !---------------------------------------------------------------------
!   subroutine makeAndSaveThumbnailFromArgs(srcXpixel, srcYpixel, image, thumbnailName&
!      , instStr, srcIndexStr, srcCatName, thumbCommon, thumbThis)
!   ! , instStr, srcIndexStr, srcCatName, thumbSize)
!   !---------------------------------------------------------------------
! 
!     implicit none 
! 
!     real(single), intent(in) ::&
!       srcXpixel, srcYpixel
!     real(single), intent(in) ::&
!       image(:,:)
!     character(*), intent(in) ::&
!       thumbnailName, instStr
!     character(*), intent(in) :: srcIndexStr
!     !integer, intent(in) :: srcIndex
!     character(*), intent(in) :: srcCatName
!     type(ThumbCommonT), intent(in) :: thumbCommon
!     ! type(ThumbSizeInfoT), intent(in) :: thumbSize
!     type(ThumbOneT),    intent(in), optional :: thumbThis
! 
!     type(ThumbSizeInfoT) :: thumbSize
!     real(single), allocatable :: thumbImage(:,:)
!     integer :: xsize, ysize, startXi, finisXi, startYi, finisYi
! 
!     !-------------------------------------------------------------------
!     ! Start:
! 
!     thumbSize =  thumbCommon%tSize
! !    if (present(thumbSize)) then
!     allocate( &
!        thumbImage(-thumbSize%halfWidthPixels:thumbSize%halfWidthPixels &
!                 , -thumbSize%halfWidthPixels:thumbSize%halfWidthPixels) )
! !    else
! !allocate( &
! !   thumbImage(-ThumbnailHalfWidthPixels:ThumbnailHalfWidthPixels&
! !            , -ThumbnailHalfWidthPixels:ThumbnailHalfWidthPixels) )
! !    end if
! 
!     xsize = size(image, 1)
!     ysize = size(image, 2)
! 
!     if (srcXpixel .lt. 1 .or. srcXpixel .gt. xsize&
!     .or. srcYpixel .lt. 1 .or. srcYpixel .gt. ysize) then
!       call warning('srcOutOfField', 'The source ('//trim(srcCatName) &
!          //') is out of the image field.  GIF file (' &
!          //trim(thumbnailName)//') is not created.')
!   return
!     end if
! 
!     thumbImage = 0.0
! !    if (present(thumbSize)) then
!     startXi = max(1,     nint(srcXpixel) - thumbSize%halfWidthPixels)
!     finisXi = min(xsize, nint(srcXpixel) + thumbSize%halfWidthPixels)
!     startYi = max(1,     nint(srcYpixel) - thumbSize%halfWidthPixels)
!     finisYi = min(ysize, nint(srcYpixel) + thumbSize%halfWidthPixels)
! !    else
! !startXi = max(1,     nint(srcXpixel) - ThumbnailHalfWidthPixels)
! !finisXi = min(xsize, nint(srcXpixel) + ThumbnailHalfWidthPixels)
! !startYi = max(1,     nint(srcYpixel) - ThumbnailHalfWidthPixels)
! !finisYi = min(ysize, nint(srcYpixel) + ThumbnailHalfWidthPixels)
! !    end if
! 
!     thumbImage(startXi-nint(srcXpixel):finisXi-nint(srcXpixel)&
!     , startYi-nint(srcYpixel):finisYi-nint(srcYpixel))&
!     = image(startXi:finisXi, startYi:finisYi)
! 
! if (present(thumbThis)) then
!     call makeThumbnailGif(thumbImage, thumbThis, thumbCommon)
! else
!     ! call makeThumbnailGif(thumbImage, thumbnailName, instStr, srcIndexStr &
!     !   , srcCatName, thumbSize, lWidth=thumbCommon%prmPgplot%lWidth &
!     !   , sizeIndexCross=real(thumbCommon%sizeIndexCross) &
!     !   , fontNumber=thumbCommon%prmPgplot%fontNumber)
!     !   ! , srcCatName, thumbSize, lWidth=ArgLWidth &
!     !   ! , sizeIndexCross=real(ArgSizeIndexCross), fontNumber=ArgFontNumber)
! end if
! 
!   end subroutine makeAndSaveThumbnailFromArgs


  !---------------------------------------------------------------------
  subroutine makeThumbnailGif(thumbImage, thumbThis, thumbCommon)
  !---------------------------------------------------------------------
  ! Main routine to create and output GIF image.

    implicit none 

    real(single), intent(in) :: thumbImage(:,:)	! The entire array is converted into GIF.
    type(ThumbOneT),    intent(in) :: thumbThis
    type(ThumbCommonT), intent(in) :: thumbCommon

    ! character(*), intent(in) ::&
    !   thumbnailName, instStr
!    real, intent(in) ::&
!      zMin, zMax
    ! character(*), intent(in) :: indexStr
    ! ! integer, intent(in) :: index
    ! character(*), intent(in) :: srcCatName
    ! type(ThumbSizeInfoT), intent(in) :: thumbSize
    ! real, intent(in), optional :: sizeIndexCross
    ! integer, intent(in), optional :: lWidth, fontNumber

    ! character(LENINSTSTRPRINT)  :: instStr
    ! character(MAXSTRINDEX) :: indexStr
    ! character(LENMAXTHUMBNAILFILENAME) :: srcCatName
    ! real :: sizeIndexCross
    ! integer :: lWidth, fontNumber

    real, parameter :: &
       trMatrix(6) = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 /)

    integer :: lWidthNow
    integer :: ierr, pgbeg, xsize, ysize
    real :: vsLoXinches, vsHiXinches, vsLoYinches, vsHiYinches,&
      widthWholeInches, widthThumbInches, aspect, xmin, xmax, ymin, ymax, zMax
!,&
!      zClipFractionHi
    ! character(10) :: indexStr
integer, parameter :: imageTransferId = 0
    ! 0: linear, 1: log, 2: sqrt
real, parameter :: fixedZmax = 6.0
    character(40) :: lowerStr
real :: l, s
integer :: winHalfDx, winHalfDy, winStartX, winFinisX, winStartY, winFinisY
real :: zLevelInner, zLevelOuter
logical, allocatable :: zMask(:,:)

    !-------------------------------------------------------------------
    ! Start:

    xsize = size(thumbImage, 1)
    ysize = size(thumbImage, 2)

    xmin = 0.5
    xmax = xsize + 0.5
    ymin = 0.5
    ymax = ysize + 0.5

    ! Sort out brightness scaling:

!    zClipFractionHi = 1.0 - Pi * radiusArcsec * radiusArcsec&
!      / thumbnailHalfWidthArcmin / thumbnailHalfWidthArcmin / 60.0 / 60.0
!
!    if (zClipFractionHi >= 1.0 .or. zClipFractionHi <= 0.0) then
!      zmax = fixedZmax
!
!    else
!      zMax = valueAtHistoFraction(real(thumbImage), zClipFractionHi)
!      ! function valueAtHistoFraction() is in ssclib/array_utils.
!!print *, 'z value at bin fraction ', zClipFractionHi, ' is ', zMax
!
!      zMax = max(fixedZmax, zMax)
!    end if

! The following algorithm divides thumbImage into a central rectangular window + the remainder. Two levels are calculated: (i) the average level in the central window (including 0-valued pixels); (ii) a histogram fraction level for the remaining pixels. The higher of these will be used for zMax (which is the image saturation value).

winHalfDx = floor(0.05 * xsize)
winHalfDy = floor(0.05 * ysize)
if (isOdd(xsize)) then
  winStartX = ceiling(xsize / 2.0) - winHalfDx
  winFinisX = ceiling(xsize / 2.0) + winHalfDx
  ! Ensures winDx always >= 1
else
  winStartX = xsize / 2 - winHalfDx
  winFinisX = xsize / 2 + 1 + winHalfDx
  ! Ensures winDx always >= 2
end if
if (isOdd(ysize)) then
  winStartY = ceiling(ysize / 2.0) - winHalfDy
  winFinisY = ceiling(ysize / 2.0) + winHalfDy
  ! Ensures winDy always >= 1
else
  winStartY = ysize / 2 - winHalfDy
  winFinisY = ysize / 2 + 1 + winHalfDy
  ! Ensures winDy always >= 2
end if

if  (winStartX <= 1 .or. winFinisX >= xsize&
.or. winStartY <= 1 .or. winFinisY >= ysize)&
call error('imageTooSmall','')

zLevelInner = sum(thumbImage(winStartX:winFinisX,winStartY:winFinisY))&
  / real(winFinisX-winStartX+1) / real(winFinisY-winStartY+1)

allocate(zMask(xsize, ysize))
zMask = .true.
zMask(winStartX:winFinisX,winStartY:winFinisY) = .false.
where(thumbImage <= 0.0) zMask = .false.

zLevelOuter = valueAtHistoFraction(real(thumbImage), 0.65, zMask) * 3.5
! function valueAtHistoFraction() is in ssclib/array_utils.

deallocate(zMask)

zMax = max(zLevelInner, zLevelOuter)

    ! Now set up PGPLOT. CGP: "Note use PGBEG not PGOPEN because latter
    ! does not get picked up from library on some installations."
    ierr = pgbeg(0, trim(thumbThis%filename)//'/gif', 1, 1)
!ierr = pgbeg(0, '/xs', 1, 1)
if(ierr .ne. 1) call error("noPgplotGifDevice", "")

    call pgqvsz(1, vsLoXinches, vsHiXinches, vsLoYinches, vsHiYinches) ! get
    ! size of view surface in inches.

    widthWholeInches = vsHiXinches - vsLoXinches
!    if (present(thumbSize)) then
    widthThumbInches = widthWholeInches &
       * (xsize / real(thumbCommon%tSize%orgImageXSize))
!    else
!widthThumbInches = widthWholeInches * (xsize / real(ImageXsize))
!    end if

widthThumbInches = widthThumbInches * 1.3
!    aspect = (vsHiYinches - vsLoYinches) / widthWholeInches
    aspect = 1.15
    call pgpap(widthThumbInches, aspect)

!    call pgsvp(0.0, 1.0, 0.0, 0.9) 
call pgvsiz(0.0, widthThumbInches, 0.0, widthThumbInches) 
!    call pgwnad(xmin, xmax, ymin, ymax)
call pgswin(xmin, xmax, ymin, ymax)
    call set_palette(thumbCommon%prmPgplot%colourMapId)
    call pgsitf(imageTransferId)
    call pgimag(real(thumbImage), xsize, ysize, 1, xsize, 1, ysize&
      , 0.0, zMax, trMatrix)

    call pgsfs(2) ! hollow
!call pgscr(3, 0.5, 1.0, 0.0)
    call pgsci(3) ! green

    lWidthNow = thumbCommon%prmPgplot%lWidth
    ! if (present(lWidth)) then
    !   lWidthNow = lWidth
    ! end if
    if (lWidthNow < 1) then
!      if (present(thumbSize)) then
      lWidthNow = nint(thumbCommon%tSize%halfWidthArcmin)	! 4 for 8 arcmin image
!      else
!lWidthNow = nint(thumbnailHalfWidthArcmin)	! 4 for 8 arcmin image
!      end if
    end if
    if ( lWidthNow < 2 ) then
      lWidthNow = 2	! a bit wider than default (no effect in GIF?)
      call warning('tooSmallLWidth' &
         , 'Lwidth for PGPLOT is set to be the minimum value, 2.')
    end if
    call pgslw(lWidthNow)

!    call pgcirc(fracVal(xmin, xmax, 0.50), fracVal(ymin, ymax, 0.50)&
!      , radiusArcsec / arcsecPerPixelX)

! draw crosshairs:
!l = 0.12 ! original
!s = 0.05 ! original
l = 0.24
s = 0
    ! if (present(sizeIndexCross)) then
    !     s = sizeIndexCross
    ! end if
    s = thumbCommon%sizeIndexCross
    if (s <= 0) then
      s = 0.3/thumbCommon%tSize%halfWidthArcmin + 0.025
      ! 0.1  for  8 arcmin image
      ! 0.05 for 24 arcmin image
    end if

call pgmove(fracVal(xmin, xmax, 0.5-l), fracVal(ymin, ymax, 0.5))
call pgdraw(fracVal(xmin, xmax, 0.5-s), fracVal(ymin, ymax, 0.5))
call pgmove(fracVal(xmin, xmax, 0.5+s), fracVal(ymin, ymax, 0.5))
call pgdraw(fracVal(xmin, xmax, 0.5+l), fracVal(ymin, ymax, 0.5))
call pgmove(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5-l))
call pgdraw(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5-s))
call pgmove(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5+s))
call pgdraw(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5+l))

    call pgsci(1) ! white
    call pgslw(1)
!    call pgbox('BC', 0.0, 0, 'BC', 0.0, 0)
call pgmove(xmin, ymax)
call pgdraw(xmax, ymax)

    ! indexStr = intToStr(index)

    ! if (present(fontNumber)) then
    !   if (fontNumber > 0) then
    !     call pgscf(fontNumber)
    !   end if
    ! !else
    !   ! call pgscf(2)			! better quality Roman font
    !   !call pgscf(1)			! Normal font
    ! end if
    if (thumbCommon%prmPgplot%fontNumber > 0) then
      call pgscf(thumbCommon%prmPgplot%fontNumber)
    else
      call message(AppMsg, Noisy, 'Font is not being explicitly set.')
    end if
    call pgsch(2.0)
    call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.10)&
      , trim(thumbThis%fullCatName)//trim(thumbThis%fullIndexStr))
      !, trim(srcCatName))
    !call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
    call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
      , 0.0, 1.0, trim(thumbCommon%commUpper))
    call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.03)&
      , 0.0, 1.0, trim(thumbCommon%commLower))
     !, 0.0, 1.0, 'XMM/SSC')
    !lowerStr = ArgObsIdStr//'/'//trim(indexStr)//' '//trim(instStr)
    lowerStr = trim(thumbCommon%obsIdStr)//' '//trim(thumbCommon%instStr) &
       //' '//trim(thumbCommon%eRangeStr)
    call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.03)&
      , trim(lowerStr))
    !call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.03)&
    !  , 0.0, 1.0, trim(ArgERangeStr))


!    call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
!      , 0.0, 1.0, trim(indexStr))
!    call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.03)&
!      , 'RRRR_OOOOOOOOOO mos1 0.5-2 keV')

    call pgend

  end subroutine makeThumbnailGif




!   !---------------------------------------------------------------------
!   subroutine makeThumbnailGif_old(thumbImage, thumbnailName, instStr, indexStr&
!   , srcCatName, thumbSize, lWidth, sizeIndexCross, fontNumber)
!   !---------------------------------------------------------------------
! 
!     implicit none 
! 
!     real(single), intent(in) ::&
!       thumbImage(:,:)
!     character(*), intent(in) ::&
!       thumbnailName, instStr
! !    real, intent(in) ::&
! !      zMin, zMax
!     character(*), intent(in) :: indexStr
!     ! integer, intent(in) :: index
!     character(*), intent(in) :: srcCatName
!     type(ThumbSizeInfoT), intent(in) :: thumbSize
!     real, intent(in), optional :: sizeIndexCross
!     integer, intent(in), optional :: lWidth, fontNumber
! 
!     ! character(LENINSTSTRPRINT)  :: instStr
!     ! character(MAXSTRINDEX) :: indexStr
!     ! character(LENMAXTHUMBNAILFILENAME) :: srcCatName
!     ! real :: sizeIndexCross
!     ! integer :: lWidth, fontNumber
! 
!     real, parameter :: &
!        trMatrix(6) = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 /)
! 
!     integer :: lWidthNow
!     integer :: ierr, pgbeg, xsize, ysize
!     real :: vsLoXinches, vsHiXinches, vsLoYinches, vsHiYinches,&
!       widthWholeInches, widthThumbInches, aspect, xmin, xmax, ymin, ymax, zMax
! !,&
! !      zClipFractionHi
!     ! character(10) :: indexStr
! integer, parameter :: imageTransferId = 0
!     ! 0: linear, 1: log, 2: sqrt
! real, parameter :: fixedZmax = 6.0
!     character(40) :: lowerStr
! real :: l, s
! integer :: winHalfDx, winHalfDy, winStartX, winFinisX, winStartY, winFinisY
! real :: zLevelInner, zLevelOuter
! logical, allocatable :: zMask(:,:)
! 
!     !-------------------------------------------------------------------
!     ! Start:
! 
!     xsize = size(thumbImage, 1)
!     ysize = size(thumbImage, 2)
! 
!     xmin = 0.5
!     xmax = xsize + 0.5
!     ymin = 0.5
!     ymax = ysize + 0.5
! 
!     ! Sort out brightness scaling:
! 
! !    zClipFractionHi = 1.0 - Pi * radiusArcsec * radiusArcsec&
! !      / thumbnailHalfWidthArcmin / thumbnailHalfWidthArcmin / 60.0 / 60.0
! !
! !    if (zClipFractionHi >= 1.0 .or. zClipFractionHi <= 0.0) then
! !      zmax = fixedZmax
! !
! !    else
! !      zMax = valueAtHistoFraction(real(thumbImage), zClipFractionHi)
! !      ! function valueAtHistoFraction() is in ssclib/array_utils.
! !!print *, 'z value at bin fraction ', zClipFractionHi, ' is ', zMax
! !
! !      zMax = max(fixedZmax, zMax)
! !    end if
! 
! ! The following algorithm divides thumbImage into a central rectangular window + the remainder. Two levels are calculated: (i) the average level in the central window (including 0-valued pixels); (ii) a histogram fraction level for the remaining pixels. The higher of these will be used for zMax (which is the image saturation value).
! 
! winHalfDx = floor(0.05 * xsize)
! winHalfDy = floor(0.05 * ysize)
! if (isOdd(xsize)) then
!   winStartX = ceiling(xsize / 2.0) - winHalfDx
!   winFinisX = ceiling(xsize / 2.0) + winHalfDx
!   ! Ensures winDx always >= 1
! else
!   winStartX = xsize / 2 - winHalfDx
!   winFinisX = xsize / 2 + 1 + winHalfDx
!   ! Ensures winDx always >= 2
! end if
! if (isOdd(ysize)) then
!   winStartY = ceiling(ysize / 2.0) - winHalfDy
!   winFinisY = ceiling(ysize / 2.0) + winHalfDy
!   ! Ensures winDy always >= 1
! else
!   winStartY = ysize / 2 - winHalfDy
!   winFinisY = ysize / 2 + 1 + winHalfDy
!   ! Ensures winDy always >= 2
! end if
! 
! if  (winStartX <= 1 .or. winFinisX >= xsize&
! .or. winStartY <= 1 .or. winFinisY >= ysize)&
! call error('imageTooSmall','')
! 
! zLevelInner = sum(thumbImage(winStartX:winFinisX,winStartY:winFinisY))&
!   / real(winFinisX-winStartX+1) / real(winFinisY-winStartY+1)
! 
! allocate(zMask(xsize, ysize))
! zMask = .true.
! zMask(winStartX:winFinisX,winStartY:winFinisY) = .false.
! where(thumbImage <= 0.0) zMask = .false.
! 
! zLevelOuter = valueAtHistoFraction(real(thumbImage), 0.65, zMask) * 3.5
! ! function valueAtHistoFraction() is in ssclib/array_utils.
! 
! deallocate(zMask)
! 
! zMax = max(zLevelInner, zLevelOuter)
! 
!     ! Now set up PGPLOT. CGP: "Note use PGBEG not PGOPEN because latter
!     ! does not get picked up from library on some installations."
!     ierr = pgbeg(0, trim(thumbnailName)//'/gif', 1, 1)
! !ierr = pgbeg(0, '/xs', 1, 1)
! if(ierr .ne. 1) call error("noPgplotGifDevice", "")
! 
!     call pgqvsz(1, vsLoXinches, vsHiXinches, vsLoYinches, vsHiYinches) ! get
!     ! size of view surface in inches.
! 
!     widthWholeInches = vsHiXinches - vsLoXinches
! !    if (present(thumbSize)) then
!     widthThumbInches = widthWholeInches * (xsize / real(thumbSize%orgImageXSize))
! !    else
! !widthThumbInches = widthWholeInches * (xsize / real(ImageXsize))
! !    end if
! 
! widthThumbInches = widthThumbInches * 1.3
! !    aspect = (vsHiYinches - vsLoYinches) / widthWholeInches
!     aspect = 1.15
!     call pgpap(widthThumbInches, aspect)
! 
! !    call pgsvp(0.0, 1.0, 0.0, 0.9) 
! call pgvsiz(0.0, widthThumbInches, 0.0, widthThumbInches) 
! !    call pgwnad(xmin, xmax, ymin, ymax)
! call pgswin(xmin, xmax, ymin, ymax)
!     call set_palette(ArgColourMapId)
!     call pgsitf(imageTransferId)
!     call pgimag(real(thumbImage), xsize, ysize, 1, xsize, 1, ysize&
!       , 0.0, zMax, trMatrix)
! 
!     call pgsfs(2) ! hollow
! !call pgscr(3, 0.5, 1.0, 0.0)
!     call pgsci(3) ! green
! 
!     lWidthNow = 0
!     if (present(lWidth)) then
!       lWidthNow = lWidth
!     end if
!     if (lWidthNow < 1) then
! !      if (present(thumbSize)) then
!       lWidthNow = nint(thumbSize%halfWidthArcmin)	! 4 for 8 arcmin image
! !      else
! !lWidthNow = nint(thumbnailHalfWidthArcmin)	! 4 for 8 arcmin image
! !      end if
!     end if
!     if ( lWidthNow < 2 ) then
!       lWidthNow = 2	! a bit wider than default (no effect in GIF?)
!     end if
!     call pgslw(lWidthNow)
! 
! !    call pgcirc(fracVal(xmin, xmax, 0.50), fracVal(ymin, ymax, 0.50)&
! !      , radiusArcsec / arcsecPerPixelX)
! 
! ! draw crosshairs:
! !l = 0.12 ! original
! !s = 0.05 ! original
! l = 0.24
! s = 0
!     if (present(sizeIndexCross)) then
!       if (sizeIndexCross > 0) then
!         s = sizeIndexCross
!       end if
!     end if
!     if (s <= 0) then
! !      if (present(thumbSize)) then
!       s = 0.3/thumbSize%halfWidthArcmin + 0.025
! !      else
! !s = 0.3/thumbnailHalfWidthArcmin + 0.025
! !      end if
!       ! 0.1  for  8 arcmin image
!       ! 0.05 for 24 arcmin image
!     end if
! 
! call pgmove(fracVal(xmin, xmax, 0.5-l), fracVal(ymin, ymax, 0.5))
! call pgdraw(fracVal(xmin, xmax, 0.5-s), fracVal(ymin, ymax, 0.5))
! call pgmove(fracVal(xmin, xmax, 0.5+s), fracVal(ymin, ymax, 0.5))
! call pgdraw(fracVal(xmin, xmax, 0.5+l), fracVal(ymin, ymax, 0.5))
! call pgmove(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5-l))
! call pgdraw(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5-s))
! call pgmove(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5+s))
! call pgdraw(fracVal(xmin, xmax, 0.5  ), fracVal(ymin, ymax, 0.5+l))
! 
!     call pgsci(1) ! white
!     call pgslw(1)
! !    call pgbox('BC', 0.0, 0, 'BC', 0.0, 0)
! call pgmove(xmin, ymax)
! call pgdraw(xmax, ymax)
! 
!     ! indexStr = intToStr(index)
! 
!     if (present(fontNumber)) then
!       if (fontNumber > 0) then
!         call pgscf(fontNumber)
!       end if
!     !else
!       ! call pgscf(2)			! better quality Roman font
!       !call pgscf(1)			! Normal font
!     end if
!     call pgsch(2.0)
!     call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.10)&
!       , trim(srcCatName)//trim(indexStr))
!       !, trim(srcCatName))
!     !call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
!     call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
!       , 0.0, 1.0, trim(ArgCommUpper))
!     call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.03)&
!       , 0.0, 1.0, trim(ArgCommLower))
!      !, 0.0, 1.0, 'XMM/SSC')
!     !lowerStr = ArgObsIdStr//'/'//trim(indexStr)//' '//trim(instStr)
!     lowerStr = trim(ArgObsIdStr)//' '//trim(instStr)//' '//trim(ArgERangeStr)
! ! where ArgObsIdStr: Global (Command-line argument --- should be replaced,
! ! i.e., should be included only in makeAndSaveThumbnail().) 
!     call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.03)&
!       , trim(lowerStr))
!     !call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.03)&
!     !  , 0.0, 1.0, trim(ArgERangeStr))
! 
! 
! !    call pgptxt(fracVal(xmin, xmax, 0.98), fracVal(ymin, ymax, 1.10)&
! !      , 0.0, 1.0, trim(indexStr))
! !    call pgtext(fracVal(xmin, xmax, 0.02), fracVal(ymin, ymax, 1.03)&
! !      , 'RRRR_OOOOOOOOOO mos1 0.5-2 keV')
! 
!     call pgend
! 
!   end subroutine makeThumbnailGif_old


  !---------------------------------------------------------------------
  subroutine set_palette(itype)
  !---------------------------------------------------------------------
  ! set_palette: Sets a color palette for use by PGIMAG.

!**** copied from implot_b and implot.

integer, intent(in) :: itype    !colour palette: -1=inverse gray, 1=grey, 2=rainbow, 3=heat, 
                                ! 4=IRAF, 5=AIPS, 6=TJP
real, parameter :: al(20) = (/0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5, &
                              0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/)
real, parameter :: ar(20) = (/0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, &
                              0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/)
real, parameter :: ag(20) = (/0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, &
                              0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/)
real, parameter :: ab(20) = (/0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, &
                              0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)
real, parameter :: grey(2) = (/0.0, 1.0/)
real, parameter :: invg(2) = (/1.0, 0.0/)

!real, parameter :: hl(5) = (/0.0, 0.2, 0.4, 0.6, 1.0/)
!real, parameter :: hr(5) = (/0.0, 0.5, 1.0, 1.0, 1.0/)
!real, parameter :: hg(5) = (/0.0, 0.0, 0.5, 1.0, 1.0/)
!real, parameter :: hb(5) = (/0.0, 0.0, 0.0, 0.3, 1.0/)
real, parameter :: hl(5) = (/0.0, 0.2, 0.4, 0.6, 1.0/)
real, parameter :: hr(5) = (/0.0, 0.6, 1.0, 1.0, 1.0/)
real, parameter :: hg(5) = (/0.0, 0.1, 0.5, 1.0, 1.0/)
real, parameter :: hb(5) = (/0.0, 0.0, 0.0, 0.3, 1.0/)

real, parameter :: rl(9) = (/-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/)
real, parameter :: rr(9) = (/0.0, 0.0, 0.0, 0.0, 0.6, 1.0, 1.0, 1.0, 1.0/)
real, parameter :: rg(9) = (/0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 0.6, 0.0, 1.0/)
real, parameter :: rb(9) = (/0.0, 0.3, 0.8, 1.0, 0.3, 0.0, 0.0, 0.0, 1.0/)
real, parameter :: tl(4) = (/0.0, 0.5, 0.5, 1.0/)
real, parameter :: tr(4) = (/0.2, 0.6, 0.6, 1.0/)
real, parameter :: tg(4) = (/0.0, 0.0, 0.5, 1.0/)
real, parameter :: tb(4) = (/1.0, 0.0, 0.0, 0.0/)
real, parameter :: wl(10) = (/0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, &
                              0.95, 1.0/)
real, parameter :: wr(10) = (/0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0, &
                              1.0, 1.0/)
real, parameter :: wg(10) = (/0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  &
                              1.0, 1.0/)
real, parameter :: wb(10) = (/0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, &
                              0.95, 1.0/)
! following designed by Clive Page: black + rainbow + white at top end
real, parameter :: cl(7) = (/0.1,  0.25, 0.43, 0.57, 0.71, 0.8,  1.0/)
real, parameter :: cr(7) = (/0.5,  0.0,  0.0,  0.0,  0.9,  1.0,  1.0/)
real, parameter :: cg(7) = (/0.0,  0.0,  0.8,  1.0,  0.9,  0.3,  1.0/)
real, parameter :: cb(7) = (/0.5,  1.0,  0.8,  0.0,  0.0,  0.3,  1.0/)
!                          black  mag   blue  cyan  grn   yell  red   white

select case(itype)
case(-2)                                             ! inverse rainbow
   call pgctab(rl, rr(9:1:-1), rg(9:1:-1), rb(9:1:-1), 9, 1.0, 0.5)
case(-1) 
   call pgctab(grey, invg, invg, invg, 2, 1.0, 0.5)  ! inverse greyscale
case(0)
call error('badColourMap', 'Sorry, --colourmap=0 is not allowed.')
case(1)
   call pgctab(grey, grey, grey, grey, 2, 1.0, 0.5)  ! greyscale
case(2)
   call pgctab(rl, rr, rg, rb, 9, 1.0, 0.5)          ! rainbow
case(3)
   call pgctab(hl, hr, hg, hb, 5, 1.0, 0.5)          ! heat
!   call pgctab(hl, hr, hg, hb, 5, 0.8, 0.6)          ! heat
case(4)
   call pgctab(wl, wr, wg, wb, 10, 1.0, 0.5)         ! weird IRAF
case(5)
   call pgctab(al, ar, ag, ab, 20, 1.0, 0.5)         ! AIPS
case(6)
   call pgctab(tl, tr, tg, tb, 4, 1.0, 0.5)          ! Tim Pearson
case(7)
   call pgctab(cl, cr, cg, cb, 8, 1.0, 0.5)         ! Clive Page
   call pgscr(16, 0.0, 0.0, 0.0)                    ! bottom level is black
end select
return
end subroutine set_palette


  !---------------------------------------------------------------------
  !  Subroutine: writenumber(NUMBER,fmt=?,var=?)
  !---------------------------------------------------------------------
  ! Write the number into STDOUT/given-variable (in the format if given).
  ! If the format is not given, but if variable is given,
  ! then preceding space(s) are omitted.  However,
  ! in the case of a positive and float (double) value,
  ! a preceding space is left (which seems to be reserved for a sign).
  ! Note that if -0.0 is given, the returned value will be something
  ! like "-0.0000000".

  !---------------------------------------------------------------------
  subroutine writeint8(i,fmt,var)
  !---------------------------------------------------------------------
    implicit none 

    integer(int8), intent(in) :: i
    character(*), intent(in), optional :: fmt
    character(*), intent(inout), optional :: var

    if (present(fmt)) then
      if (present(var)) then
        write(var,fmt) i
      else
        write(*,fmt) i
      end if
    else
      if (present(var)) then
        write(var,*) i
        var = var(2:)		! Omitting the preceeding single space
        var = ADJUSTL(var)
      else
        write(*,*) i
      end if
    end if

  end subroutine writeint8

  !---------------------------------------------------------------------
  subroutine writeint16(i,fmt,var)
  !---------------------------------------------------------------------
    implicit none 

    integer(int16), intent(in) :: i
    character(*), intent(in), optional :: fmt
    character(*), intent(inout), optional :: var

    if (present(fmt)) then
      if (present(var)) then
        write(var,fmt) i
      else
        write(*,fmt) i
      end if
    else
      if (present(var)) then
        write(var,*) i
        var = var(2:)		! Omitting the preceeding single space
        var = ADJUSTL(var)
      else
        write(*,*) i
      end if
    end if

  end subroutine writeint16

  !---------------------------------------------------------------------
  subroutine writeint32(i,fmt,var)
  !---------------------------------------------------------------------
    implicit none 

    integer(int32), intent(in) :: i
    character(*), intent(in), optional :: fmt
    character(*), intent(inout), optional :: var

    if (present(fmt)) then
      if (present(var)) then
        write(var,fmt) i
      else
        write(*,fmt) i
      end if
    else
      if (present(var)) then
        write(var,'(I5)') i
        var = var(2:)		! Omitting the preceeding single space
        var = ADJUSTL(var)
      else
        write(*,*) i
      end if
    end if
  end subroutine writeint32

  !---------------------------------------------------------------------
  subroutine writereal(r,fmt,var)
  !---------------------------------------------------------------------
    implicit none 

    real(single), intent(in) :: r
    character(*), intent(in), optional :: fmt
    character(*), intent(inout), optional :: var

    if (present(fmt)) then
      if (present(var)) then
        write(var,fmt) r
      else
        write(*,fmt) r
      end if
    else
      if (present(var)) then
        write(var,*) r
        ! var = var(3:)		! Omitting the preceeding double space
        call writenumber_deleteSpaces(var)
      else
        write(*,*) r
      end if
    end if

  end subroutine writereal

  !---------------------------------------------------------------------
  subroutine writedouble(r,fmt,var)
  !---------------------------------------------------------------------
    implicit none 

    real(double), intent(in) :: r
    character(*), intent(in), optional :: fmt
    character(*), intent(inout), optional :: var

    if (present(fmt)) then
      if (present(var)) then
        write(var,fmt) r
      else
        write(*,fmt) r
      end if
    else
      if (present(var)) then
        write(var,*) r
        ! var = var(3:)		! Omitting the preceeding double space
        call writenumber_deleteSpaces(var)
      else
        write(*,*) r
      end if
    end if

  end subroutine writedouble


  !---------------------------------------------------------------------
  subroutine writenumber_deleteSpaces(str)
  !---------------------------------------------------------------------
  ! Private.
  ! Delete preceding spaces in writenumber().

    implicit none 

    character(*), intent(inout) :: str

    integer(int32) :: i,iFini

    !-------------------------------------------------------------------
    ! Start:

! if (.true.) then
    iFini = len_trim(str) 
    do i=1, iFini
      if (str(i:i) .ne. ' ') exit
    end do
    if (i > iFini) then
      str = ''
    else
      str = trim(str(i:))
    end if
! else
    ! !! Obsolete routine
    ! do
    !   if (str(1:1) == ' ') then
    !     str = str(2:)
    !   else
    !     return
    !   end if
    ! end do
! end if

  end subroutine writenumber_deleteSpaces


  !---------------------------------------------------------------------
  subroutine retDirname(path, dirName)
  !---------------------------------------------------------------------
  ! Return the directory name from the given path.
  !   /a/b/c  => /a/b
  !   /a/b/c/ => /a/b/c
  ! Note: Preceding spaces are NOT deleted.

    implicit none 

    character(*), intent(in)    :: path
    character(*), intent(inout) :: dirName
    integer(int32) :: i32

    !-------------------------------------------------------------------
    ! Start:

    do i32=len_trim(path),1,-1
      if (path(i32:i32) == '/') then
        exit
      end if
    end do

    dirName = path(1:i32-1)

  end subroutine retDirname


  !---------------------------------------------------------------------
  function isDirectoryReadable(path) result(retBool)
  !---------------------------------------------------------------------
  ! True if a given directory is readable.

    implicit none 

    logical :: retBool
    character(*), intent(in) :: path

    logical :: doesExist
    character(16) :: strIsReadable

    !-------------------------------------------------------------------
    ! Start:

    inquire(file=trim(path) &
          , exist=doesExist &
          , read=strIsReadable)

    if (doesExist .and. (strIsReadable .ne. 'NO')) then
      retBool = .true.
    else
      retBool = .false.
    end if

  end function isDirectoryReadable


  !---------------------------------------------------------------------
  function isDirnameReadable(path) result(retBool)
  !---------------------------------------------------------------------
  ! True if the directory in the given path is readable.
  ! This is simply a combination of retDirname() with isDirectoryReadable().

    implicit none 

    logical :: retBool
    character(*), intent(in) :: path

    character(4096) :: dirName

    !-------------------------------------------------------------------
    ! Start:

    call retDirname(path, dirName)
    if (isDirectoryReadable(trim(dirName))) then
      retBool = .true.
      call message(AppMsg, Noisy &
         , 'Directory ('//trim(dirName)//') is readable at least, ' &
         //'though is not guaranteed to be executable (writable in it).')
    else
      retBool = .false.
      call message(AppMsg, Noisy &
         , 'Directory ('//trim(dirName)//') either does not exist ' &
         //'or is not readable.')
    end if

  end function isDirnameReadable


  !---------------------------------------------------------------------
  subroutine dumpThumbOne(thumbOne, varName, header, footer)
  !---------------------------------------------------------------------
  ! Dump a ThumbOneT type variable.

    implicit none 

    type(ThumbOneT), intent(in) :: thumbOne
    character(*), intent(in), optional :: varName, header, footer

    !-------------------------------------------------------------------
    ! Start:

    if (present(varName)) then
      print *, '========== Variable ', trim(varName), ' =========='
    else if (present(header)) then
      if (trim(header) == '') then
        ! Do nothing
      else
        print *, trim(header)
      end if
    else
      print *, '========== ThumbOneT variable =========='
    end if
    print *, ' filename: (', trim(thumbOne%filename), ')'
    print *, ' (raDeg, decDeg): (', thumbOne%raDeg, ', ', thumbOne%decDeg, ')'
    print *, ' srcIndex: ', thumbOne%srcIndex
    print *, ' indexStr:     (', trim(thumbOne%indexStr), ')'
    print *, ' fullIndexStr: (', trim(thumbOne%fullIndexStr), ')'
    print *, ' detid:    ', thumbOne%detid
    print *, ' srcid:    ', thumbOne%srcid
    print *, ' (xPixel, yPixel): (', thumbOne%xPixel, ', ', thumbOne%yPixel, ')'
    print *, ' catName:     (', trim(thumbOne%catName), ')'
    print *, ' fullCatName: (', trim(thumbOne%fullCatName), ')'
    print *, ' inCatFlag: ', thumbOne%inCatFlag
    if (present(footer)) then
      if (trim(footer) == '') then
        ! Do nothing
      else
        print *, trim(footer)
      end if
    else
      print *, '===================='
    end if

  end subroutine dumpThumbOne


  !---------------------------------------------------------------------
  subroutine dumpThumbCommon(thumbCommon, varName, header, footer)
  !---------------------------------------------------------------------
  ! Dump a ThumbCommonT type variable.

    implicit none 

    type(ThumbCommonT), intent(in) :: thumbCommon
    character(*), intent(in), optional :: varName, header, footer

    !-------------------------------------------------------------------
    ! Start:

    if (present(varName)) then
      print *, '========== Variable ', trim(varName), ' =========='
    else if (present(header)) then
      if (trim(header) == '') then
        ! Do nothing
      else
        print *, trim(header)
      end if
    else
      print *, '========== ThumbCommonT variable =========='
    end if
    print *, ' orgImage(max,min): (', maxval(thumbCommon%orgImage) &
       , ', ', minval(thumbCommon%orgImage), ')'
    print *, ' instStr: (',  trim(thumbCommon%instStr), ')'
    write(*,'(A,I10)') '  obsId:    ', thumbCommon%obsId 
    print *, ' obsIdStr: ',  trim(thumbCommon%obsIdStr)
    print *, ' eRangeStr: ', trim(thumbCommon%eRangeStr)
    print *, ' commLower: ', trim(thumbCommon%commLower)
    print *, ' commUpper: ', trim(thumbCommon%commUpper)
    print *, ' sizeIndexCross: ', thumbCommon%sizeIndexCross
    print *, ' (tSize%orgImageXSize, tSize%orgImageYSize): (' &
         , thumbCommon%tSize%orgImageXSize, ', ' &
         , thumbCommon%tSize%orgImageYSize, ')'
    print *, ' tSize%halfWidthPixels: ', thumbCommon%tSize%halfWidthPixels
    print *, ' tSize%halfWidthArcmin: ', thumbCommon%tSize%halfWidthArcmin
    print *, ' prmPgplot%lWidth:      ', thumbCommon%prmPgplot%lWidth
    print *, ' prmPgplot%fontNumber:  ', thumbCommon%prmPgplot%fontNumber
    print *, ' prmPgplot%colourMapId: ', thumbCommon%prmPgplot%colourMapId
    if (present(footer)) then
      if (trim(footer) == '') then
        ! Do nothing
      else
        print *, trim(footer)
      end if
    else
      print *, '===================='
    end if

  end subroutine dumpThumbCommon


end module makethumbs_aux
