! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
! Module: makethumbs_mod
!
! Authors:  Ian Stewart - SSC/LUX
!           Masaaki Sakano - SSC/LUX 
!          
! Description: This task creates thumbnail gifs around source locations.
! 
! Comments: 
! 
! 2006/06 : XMMSRCNAME -> IAUNAME, RA -> RA_CORR, DEC -> DEC_CORR, 
! 2007/05/24 : RA/DEC is default (_NOT_ RA_CORR/DEC_CORR because the image frame is based on the uncorrected coordinates.)
! 2008/05 : Librarisation.  Inrtoduce/change the command-line parameters.
!
! TODO: 
!  * Currently SRC_NUM is assumed to be equivalent to the row number.  Not good!
!  * Accept the list of candidate column names for IAUNAME etc.
!  * Eliminate the global variable ArgTask
! 
!- 
module makethumbs_mod

  use dal
  use cal
  use param  
  use errorHandling
  use datafilter

  use dal_aux
  use wcs_aux
  use constants
  use coordinate
  use ssc_misc
  use array_utils
  use cal_aux

  use makethumbs_aux
  use ingestsrcnames_aux

  implicit none 

  private 

  ! integer, parameter :: numObsIdStr = 10
  ! character(6), parameter :: fmtDetIdStr='(I6.6)'

  ! Command-line parameters:
  type :: ArgTaskT
    character(LENINPUTFILENAME) :: &	! defined in ingestsrcnames_aux
       imageSetName, &
       srcListSetTabName, &
       ! selectionExpression,&
       gifNameRoot, &
       gifSuffix, &
       iaunamePrefix, &
       fnameStyle, &
       srcIndexSeparator, &
       srcIndexStyle, &
       catSetTabName, &
       outGifName
    character(4)                :: fnameSeparator
    character(LENOBSIDSTRPRINT) :: obsIdStr
    character(LENINSTSTRPRINT)  :: eRangeStr, instStr
    character(LENCOMMSTRPRINT)  :: commLower, commUpper
    real(double) :: &
       imageSize, &
       sizeIndexCross, &
       ra, &
       dec
    integer :: &
       srcNum = 0 ! , &
       ! lWidth, &
       ! fontNumber, &
       ! colourMapId	! 3 = heat
    logical :: &
       withSrcList, &
       withCatSet, &
       withSrcNum, &
       withCoords, &
       isAutoFname, &
       doPrintParams, &
       isDryRun
    type(PgplotDisplayPrmT) :: prmPgplot	! defined in makethums_aux
  end type ArgTaskT

  type(ArgTaskT), save :: ArgTask	! "save" here seems to be mandatory.

  ! character(LENINPUTFILENAME) ::&	! defined in ingestsrcnames_aux
  !   ArgImageSetName, &
  !   ArgSrcListSetTabName, &
  !   ! ArgSelectionExpression,&
  !   ArgGifNameRoot, &
  !   ArgGifSuffix, &
  !   ArgIaunamePrefix, &
  !   ArgFnameStyle, &
  !   ArgSrcIndexSeparator, &
  !   ArgSrcIndexStyle, &
  !   ArgCatSetTabName, &
  !   ArgOutGifName
  ! !character(numObsIdStr) ::&	! defined in makethum_aux
  ! !  ArgObsIdStr
  ! !character(15) ::&
  ! !  ArgERangeStr
  ! character(3) :: &
  !   ArgFnameSeparator
  ! real(double) :: &
  !   ArgImageSize, & ! , ArgSizeIndexCross	! defined in makethum_aux
  !   ArgRa, &
  !   ArgDec
  ! integer :: &
  !   ArgSrcNum = 0 ! , ArgLWidth, ArgFontNumber	! defined in makethum_aux
  ! logical :: &
  !   ArgIsDryRun, &
  !   ArgWithSrcList, &
  !   ArgWithCatSet, &
  !   ArgWithSrcNum, &
  !   ArgWithCoords, &
  !   ArgIsAutoFname

  public makethumbs

contains 

  !---------------------------------------------------------------------
  subroutine makethumbs
  !---------------------------------------------------------------------

    implicit none 

    real(single), pointer :: image(:,:)
    ! real(double), pointer ::&
    !   srcRaDeg(:), srcDecDeg(:)
    ! integer(int32), pointer :: srcIndex(:), srcDetid(:), srcSrcid(:), srcIndexForPrint(:)
    ! character(MAXSTRINDEX), allocatable :: srcIndexForPrintStr(:)	! Parameter defined in makethumb_aux
    ! character(LENMAXTHUMBNAILFILENAME), pointer :: srcCatName(:)
    character(LENSTRFILENAME) :: srcListSetName	! LEN... defined in ingestsrcnames_aux
    character(LENONEFITSLINE) :: srcListTabName	! LEN... defined in ingestsrcnames_aux
    logical :: noColonFoundSrcList
    ! real(single), allocatable ::&
    !   srcXpixels(:), srcYpixels(:)
    integer :: iSrc, iSrcBegin, iSrcEnd, numSources, numFilesFailed
    character(LENMAXTHUMBNAILFILENAME) :: fName	! Parameter defined in makethumb_aux
    ! character(LENINSTSTRPRINT) :: instStr	! for thumnail images
    ! character(255) :: instrumentName	! for reading the FITS header
    ! character(255) :: obsIdFromHeader
    ! logical(bool), pointer :: srcInCatFlag(:)
    type(DataSetT) :: imageSet, sourceSet
    type(TableT) :: sourceTab
    integer(int32) :: obsIdInt32	! , i32
    type(ThumbOneT), allocatable :: thumbOnes(:)	! Defined in makethumb_aux
    type(ThumbCommonT) :: thumbCommon	! Type defined in makethumb_aux
    type(ThumbSizeInfoT) :: thumbSize	! Type defined in makethumb_aux
    type(FilenameStyleT) :: outFilenameStyle	! Defined in makethumb_aux
!!logical, parameter :: isNewCodeStyle = .false.
!logical, parameter :: isNewCodeStyle = .true.


    !-------------------------------------------------------------------
    ! Start:

    call readParameters(outFilenameStyle)

    !-------------------------------------------------------------------
    ! Read the image
    !-------------------------------------------------------------------
    !if (ArgImageSize < 0.0) then
    if (ArgTask%imageSize < 0.0) then
       call error('imageTooSmall','ImageSize value has to be positive.')
    end if
! ThumbnailHalfWidthArcmin = ArgImageSize/2.0 	! Global var.
    !thumbSize%halfWidthArcmin = ArgImageSize/2.0
    thumbSize%halfWidthArcmin = ArgTask%imageSize/2.0

    call message(AppMsg, Verbose, 'Reading image ('//trim(ArgTask%imageSetName)//').')
    imageSet = dataSet(ArgTask%imageSetName, READ)

    call readArrayData(imageSet, image)
    thumbSize%orgImageXSize = size(image, 1)
    thumbSize%orgImageYSize = size(image, 2)
    allocate(thumbCommon%orgImage(thumbSize%orgImageXSize, thumbSize%orgImageYSize))
    thumbCommon%orgImage = image
! imageXsize = size(image, 1)
! imageYsize = size(image, 2)
    deallocate(image)


    if (trim(ArgTask%instStr) == 'DEFAULT') then
      thumbCommon%instStr = trim( getInstStr(imageSet) )	! in makethumb_aux
      call message(AppMsg, Verbose, 'INSTRUMENT = ('//trim(thumbCommon%instStr)//').')
    else	! if (trim(Task%irgInstStr) == '') then
      thumbCommon%instStr = trim(ArgTask%instStr)
      call message(AppMsg, Noisy,   'INSTRUMENT = ('//trim(thumbCommon%instStr)//').')
    end if	! if (trim(Task%irgInstStr) == '') then

    thumbCommon%eRangeStr = trim(ArgTask%eRangeStr)
    thumbCommon%commLower = trim(ArgTask%commLower)
    thumbCommon%commUpper = trim(ArgTask%commUpper)
    thumbCommon%sizeIndexCross = ArgTask%sizeIndexCross

    thumbCommon%prmPgplot = ArgTask%prmPgplot

    !-------------------------------------------------------------------
    ! Check ArgTask%obsIdStr(Global var.) and set it if not set.
    ! Plus, set the Integer32 version of ObsId.
    !-------------------------------------------------------------------
    if (trim(ArgTask%obsIdStr) == 'DEFAULT' .or. trim(ArgTask%obsIdStr) == '') then
      ! Get ObsId(Integer32) and then set OBS_ID(String)
      obsIdInt32 = getObsIdInt32FromHeader(imageSet)	! in ingestsrcnames_aux
      write(ArgTask%obsIdStr, FMTOBSIDSTR) obsIdInt32	! FMT... defined in makethumb_aux, getObs... defined in ingestsrcnames_aux
      call message(AppMsg, Verbose, 'OBS_ID = ('//trim(ArgTask%obsIdStr)//').')
    else
      ! Define Integer32 version of ObsId
      read(ArgTask%obsIdStr, *) obsIdInt32
      call message(AppMsg, Noisy,   'OBS_ID = ('//trim(ArgTask%obsIdStr)//').')
    end if
    write(errStr, '(A,I10)') 'OBSID  =  ', obsIdInt32
    call message(AppMsg, Noisy)

    thumbCommon%obsIdStr = ArgTask%obsIdStr
    thumbCommon%obsId    = obsIdInt32


    !-------------------------------------------------------------------
    ! Load the source data from the source list and/or external catalogue.
    !-------------------------------------------------------------------
    if (ArgTask%withSrcList) then
      !-- Open the source list.
      call splitSetTabName(ArgTask%srcListSetTabName, srcListSetName, srcListTabName &
         , noColonFoundSrcList)	! in ssclib/dal_aux
  
      if (verbosity(AppMsg) >= Noisy) then
        call message(AppMsg, Noisy, 'SRC-List name is '//trim(srcListSetName))
        if (.not. noColonFoundSrcList) then
          call message(AppMsg, Noisy &
             , 'SRC-List Table name given is '//trim(srcListTabName))
        else
          call message(AppMsg, Noisy &
             , 'SRC-List Table name to be accessed is '//trim(srcListTabName))
          ! call message(AppMsg, Noisy, 'No SRC-List Table name is given (Def: '&
          !    //trim(DEFAULTSRCTBLNAME)//').')
        end if
      end if
      sourceSet = dataSet(srcListSetName, READ)
      sourceTab = table(sourceSet, srcListTabName, READ)
      numSources = numberOfRows(sourceTab)
      allocate(thumbOnes(numSources))

      if (ArgTask%withCatSet) then
        call loadSources(sourceTab, ArgTask%srcListSetTabName &
           , ArgTask%srcIndexStyle, outFilenameStyle, thumbOnes &
           , obsIdInt32, inCatSetTabName=ArgTask%catSetTabName &
           , srcNum=ArgTask%srcNum)
      else
        call loadSources(sourceTab, ArgTask%srcListSetTabName &
           , ArgTask%srcIndexStyle, outFilenameStyle, thumbOnes &
           , obsIdInt32 &
           , srcNum=ArgTask%srcNum)
      end if

      call release(sourceTab)
      call release(sourceSet)

    else	! if (ArgTask%withSrcList) then

      numSources = 1
      allocate(thumbOnes(1))

      call message(AppMsg, Verbose, 'Loading source(s).')

      call loadSources( &
           ArgTask%srcIndexStyle, outFilenameStyle, thumbOnes(1) &
         , obsIdInt32, ArgTask%withCoords, ArgTask%withCatSet, ArgTask%withSrcNum &
         , raDeg=ArgTask%ra, decDeg=ArgTask%dec &
         , inCatSetTabName=ArgTask%catSetTabName, srcNum=ArgTask%srcNum)
    end if

    thumbOnes%fullIndexStr = trim(ArgTask%srcIndexSeparator)//thumbOnes%indexStr
    thumbOnes%fullCatName  = trim(ArgTask%iaunamePrefix)//thumbOnes%catName
    thumbCommon%tSize = thumbSize


    !-------------------------------------------------------------------
    ! Calculating the image pixel coordinates for each thumnail image.
    !-------------------------------------------------------------------
    call message(AppMsg, Verbose, 'Calculating the image coordinates for each thumbnail.')
    call calcSrcImageCoords(imageSet, thumbOnes, thumbCommon)

    call release(imageSet)

    if (ArgTask%doPrintParams) then
      call dumpThumbCommon(thumbCommon, header='========== Common parameters ==========')	! in makethumb_aux
    else if (verbosity(AppMsg) >= Noisy) then
      call dumpThumbCommon(thumbCommon, varName='thumbCommon')	! in makethumb_aux
    end if


    !-------------------------------------------------------------------
    ! MAIN: Determine each filename and make thumbnail GIFs.
    !-------------------------------------------------------------------
    ! if (ArgTask%withSrcNum .and. ArgTask%withSrcList) then
    if (ArgTask%withSrcList) then
      if ( ArgTask%srcNum <= 0 ) then
        iSrcBegin = 1
        iSrcEnd = numSources
      else
        iSrcBegin = ArgTask%srcNum
        iSrcEnd = ArgTask%srcNum
      end if
    else
      iSrcBegin = 1
      iSrcEnd = 1
      ! iSrcEnd = numSources	! 1 if the source list is not specified.
    end if

    write(errStr, '(A, I3, A, I3, A)') 'SRC_NUMs from (' &
       , iSrcBegin, ') to (', iSrcEnd, ') are processed.'
    call message(AppMsg, Noisy)

!!!!!!!!!!
!! NOTE: So far, (thumbOnes(iSrc)%srcIndex == iSrc) is assumed a priori.
!!   However, there is no guarantee for it, even though it must be
!!   always the case for the pipe-line products.
!!!!!!!!!!
    numFilesFailed = 0
    if (ArgTask%isDryRun) then
      !!! For the sake of the speed, I copy the following into the else clause.
      do iSrc = iSrcBegin, iSrcEnd
        if (.not. thumbOnes(iSrc)%inCatFlag) then
          write(errStr, '(A, I3, A)') 'Src-No (', iSrc, ') is skipped due to INCATFLG.'
          call message(AppMsg, Verbose)
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if

        if (ArgTask%isAutoFname) then
          fName = getThumbnailName(thumbOnes(iSrc), outFilenameStyle)
          ! fName = getThumbnailName(thumbOnes(iSrc)%srcIndex &
          !    , thumbOnes(iSrc)%detid, outFilenameStyle)
        else
          fName = trim(ArgTask%outGifName)
        end if
        if (isFileExistingAndQuit(trim(fName))) then
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if
        thumbOnes(iSrc)%filename = trim(fName)

        if ( .not. isSrcWithinFOV(thumbOnes(iSrc), thumbCommon) ) then
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if

        call message(AppMsg, Noisy, 'Creating GIF file ('//trim(fName)//').')

        !! Dryrun
        if (verbosity(AppMsg) >= Noisy) call dumpThumbOne(thumbOnes(iSrc))
        call message(AppMsg, Noisy, 'Created GIF file is:')
        write(*,'(A)') trim(fName)	! Write the filename

        if (ArgTask%doPrintParams .or. (verbosity(AppMsg) >= Noisy)) then
          call dumpThumbOne(thumbOnes(iSrc),header='')	! in makethumb_aux
        end if
      end do

    else
      do iSrc = iSrcBegin, iSrcEnd
        if (.not. thumbOnes(iSrc)%inCatFlag) then
          write(errStr, '(A, I3, A)') 'Src-No (', iSrc, ') is skipped due to INCATFLG.'
          call message(AppMsg, Verbose)
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if

        if (ArgTask%isAutoFname) then
          fName = getThumbnailName(thumbOnes(iSrc), outFilenameStyle)
          ! fName = getThumbnailName(thumbOnes(iSrc)%srcIndex &
          !    , thumbOnes(iSrc)%detid, outFilenameStyle)
        else
          fName = trim(ArgTask%outGifName)
        end if
        if (isFileExistingAndQuit(trim(fName))) then
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if
        thumbOnes(iSrc)%filename = trim(fName)

        if ( .not. isSrcWithinFOV(thumbOnes(iSrc), thumbCommon) ) then
          numFilesFailed = numFilesFailed + 1
          cycle	! ----------------------
        end if

        call message(AppMsg, Noisy, 'Creating GIF file ('//trim(fName)//').')

        call makeAndSaveThumbnail(thumbOnes(iSrc), thumbCommon)	! in makethumb_aux
        ! call makeAndSaveThumbnail(srcXpixels(iSrc), srcYpixels(iSrc) &
        !      , thumbCommon%orgImage, fName, thumbCommon%instStr &
        !      , trim(ArgSrcIndexSeparator)//srcIndexForPrintStr(iSrc) &
        !      , trim(ArgIaunamePrefix)//srcCatName(iSrc) &
        !      , thumbSize)
          !! If ArgSrcIndexSeparator==' ', the space is not taken into account!!!
             ! , ArgSrcIndexSeparator//srcIndexForPrintStr(iSrc), "2XMMp "//srcCatName(iSrc))
             ! , image, fName, instStr, int(srcIndexForPrint(iSrc)), srcCatName(iSrc))

        call message(AppMsg, Noisy, 'Created GIF file is:')
        write(*,'(A)') trim(fName)	! Write the filename

        if (ArgTask%doPrintParams .or. (verbosity(AppMsg) >= Noisy)) then
          call dumpThumbOne(thumbOnes(iSrc),header='')	! in makethumb_aux
        end if
      end do
    end if

    deallocate(thumbOnes)	! Should be automatically done, but just to play safe.
    if (numFilesFailed == iSrcEnd-iSrcBegin+1) then
      call error('allFilesFailed', 'Failed in creating every output file.')
    end if

  end subroutine makethumbs


  !---------------------------------------------------------------------
  subroutine readParameters(filenameStyle)
  !---------------------------------------------------------------------
  ! There are four major cases.
  !  1)  withsrclist=yes withcat=yes/no (Default)
  !  2a) withsrclist=no  withcat=yes/no withsrcnum=yes withcoords=yes
  !  2b) withsrclist=no  withcat=no     withsrcnum=no  withcoords=yes
  !  2c) withsrclist=no  withcat=yes    withsrcnum=yes withcoords=no
  ! In the case 1, multiple GIF files may be produced.
  !  Source positions are taken from the input Source-List.
  ! In the case 2a and 2b, the source position is explicitly given
  !  in the command-line.  If SRC_NUM is also given, then the catalogue
  !  may be specified, too.
  ! In the case 2c, the source is picked up from the catalogue, and
  !  so is its position (coordinates), based on SRC_NUM given in the
  !  command-line and OBS_ID in the header of the input image file.

    implicit none 

    type(FilenameStyleT), intent(out) :: filenameStyle

    character(1) :: tmpSeparator
    logical :: isSrcIndexSeparatorDef

    !-------------------------------------------------------------------
    ! Start:

    ArgTask%imageSetName      = stringParameter('imageset')
    ArgTask%withSrcList       = booleanParameter('withsrclist')
    if (ArgTask%withSrcList) then
      ArgTask%srcListSetTabName = stringParameter('srclisttab')
    else
      ArgTask%withCoords = booleanParameter('withcoords')
    end if

    ArgTask%withSrcNum        = booleanParameter('withsrcnum')
    if (ArgTask%withSrcNum .or. ArgTask%withSrcList) then
      ! In the case of the latter, reads just the default value (=0).
      ArgTask%srcNum            = intParameter('srcnum')
      ! ArgTask%srcNum            = intParameter('srcidnumber')
    else
      ArgTask%srcNum            = -999
    end if

    ! ArgTask%selectionExpression = stringParameter('expression')
!    zClipFractionLo = realParameter('zlo')
!    zClipFractionHi = realParameter('zhi')
!    withGifNameRoot = booleanParameter('withgifroot')

    ArgTask%withCatSet = booleanParameter('withcat')
    if (ArgTask%withCatSet) then
      ArgTask%catSetTabName = stringParameter('cattab')
    end if

    if (.not. ArgTask%withSrcList) then	! Single user-specified source
      if (ArgTask%withCatSet) then
        if (.not. ArgTask%withSrcNum) then
          call error('NoSrcNumSpecified' &
             , 'srcnum is mandatory when withsrclist=no and withcat=yes.')
        end if
      else if (.not. ArgTask%withCoords) then
        call error('NoCoordsSpecified' &
           , 'ra and dec are mandatory when withsrclist=no and withcat=no.')
      end if

      if (ArgTask%withCoords) then
        ! RA and DEC can be read, irrelevant to the value of withcat.
        ArgTask%ra  = realParameter('ra')
        ArgTask%dec = realParameter('dec')
      end if
    end if

    ArgTask%isAutoFname = booleanParameter('autofname')
    if (ArgTask%isAutoFname) then
      ArgTask%gifNameRoot       = stringParameter('gifroot')
      ArgTask%fnameStyle        = stringParameter('fnamestyle')
      select case(trim(ArgTask%fnameStyle))
      case('srcnum')
        filenameStyle%srcNum = .true.
        ! filenameWithSrcNum = .true.	! was defined in makethumbs_aux
      case('hexsrcnum')
        filenameStyle%srcNumHex = .true.
        ! filenameWithSrcNumHex = .true.
      case('detid')
        filenameStyle%detId  = .true.
        ! filenameWithDetId  = .true.
      case('srcid')
        filenameStyle%srcId  = .true.
        ! filenameWithSrcId  = .true.
      case('srcnum_detid')
        filenameStyle%srcNum = .true.
        filenameStyle%detId  = .true.
        ! filenameWithSrcNum = .true.
        ! filenameWithDetId  = .true.
      case('srcnum_srcid')
        filenameStyle%srcNum = .true.
        filenameStyle%srcId  = .true.
        ! filenameWithSrcNum = .true.
        ! filenameWithSrcId  = .true.
      case('all')
        filenameStyle%srcNum = .true.
        filenameStyle%srcNumHex = .true.
        filenameStyle%detId  = .true.
        filenameStyle%srcId  = .true.
        ! filenameWithSrcNum = .true.
        ! filenameWithSrcNumHex = .true.
        ! filenameWithDetId  = .true.
        ! filenameWithSrcId  = .true.
      case default
        call error('badArgument', 'Inappropriate fnamestyle option is given.')
      end select

      ! filenameWithSrcNum    = booleanParameter('fnamewithsrcnum')
      ! filenameWithSrcNumHex = booleanParameter('fnamewithsrcnumhex')
      ! filenameWithDetId     = booleanParameter('fnamewithdetid')
      ! if ((.not. filenameWithSrcNum) .and. (.not. filenameWithDetId) &
      !     .and. (.not. filenameWithSrcNumHex)) then
      !   call error('badArgument', &
      !        'Either fnamewithsrcnum or fnamewithdetid (or both) has to be yes.')
      ! end if
      ArgTask%fnameSeparator    = stringParameter('fnameseparator')
      ArgTask%gifSuffix         = stringParameter('gifsuffix')
      if (ArgTask%gifSuffix(1:1) .ne. '.') then
        ! Add '.' before the suffix if missing.
        ArgTask%gifSuffix = '.'//ArgTask%gifSuffix(1:len_trim(ArgTask%gifSuffix))
        call message(AppMsg, Verbose, 'Added preceding "." in the GIF suffix: ('&
           //trim(ArgTask%gifSuffix)//').')
      end if

    else
      ArgTask%outGifName = stringParameter('outfilename')
    end if	! if (ArgTask%isAutoFname) then

    ArgTask%srcIndexSeparator = stringParameter('srcindexseparator')
    if (trim(ArgTask%srcIndexSeparator) == 'DEFAULT') then
      isSrcIndexSeparatorDef = .true.
    else
      isSrcIndexSeparatorDef = .false.
    end if

    ! printDetId  = booleanParameter('printdetid')
    ArgTask%srcIndexStyle     = stringParameter('srcindexstyle')

    if (trim(ArgTask%srcIndexStyle) == 'DEFAULT') then
      if (ArgTask%withCatSet) then
        ArgTask%srcIndexStyle = 'detid'
      else if (ArgTask%withSrcList) then
        ArgTask%srcIndexStyle = 'srcnum'
      else
        ArgTask%srcIndexStyle = 'none'
      end if
    end if

    select case(trim(ArgTask%srcIndexStyle))
    case('none')
      tmpSeparator = ''
    case('srcnum')
      tmpSeparator = '/'
    ! case('hexsrcnum')
    !   call error('unsupportedArgument' &
    !      , 'srcindexstyle=hexsrcnum is not yet supported.  Sorry!')
    case('detid')
      tmpSeparator = ':'
    case('srcid')
      tmpSeparator = '/'
    case default
      call error('badArgument', 'Inappropriate printindexstyle option is given.')
    end select

    if (isSrcIndexSeparatorDef) then
      ArgTask%srcIndexSeparator = trim(tmpSeparator)
    end if

!    imageTransferStyle = stringParameter('scaletype')

!    select case(imageTransferStyle)
!    case('linear')
!      imageTransferId = 0
!    case('log')
!      imageTransferId = 1
!    case('sqrt')
!      imageTransferId = 2
!    case default
!call error('badImageTransferId','')
!    end select

    ArgTask%iaunamePrefix     = stringParameter('iaunameprefix')
    ArgTask%obsIdStr    = stringParameter('obsidstr')
    ArgTask%eRangeStr   = stringParameter('erangestr')
    ArgTask%instStr     = stringParameter('inststr')
    ArgTask%commLower   = stringParameter('commlower')
    ArgTask%commUpper   = stringParameter('commupper')

    ArgTask%imageSize      = realParameter('imagesize')
    ArgTask%sizeIndexCross = realParameter('sizeratiocross')

    ArgTask%prmPgplot%lWidth         = intParameter('lwidth')
    ArgTask%prmPgplot%fontNumber     = intParameter('fontnumber')
    ArgTask%prmPgplot%colourMapId    = intParameter('colourmapid')	! Default: 3 = heat
    ! ArgTask%prmPgplot%colourMapId = intParameter('colourmap')

    ArgTask%doPrintParams = booleanParameter('printparams')
    ArgTask%isDryRun      = booleanParameter('dryrun')



!     !-------------------------------------------------------------------
!     ! Start:
! 
!     ArgImageSetName      = stringParameter('imageset')
!     ArgWithSrcList       = booleanParameter('withsrclist')
!     if (ArgWithSrcList) then
!       ArgSrcListSetTabName = stringParameter('srclisttab')
!     else
!       ArgWithCoords = booleanParameter('withcoords')
!     end if
!     ArgImageSetName      = stringParameter('imageset')
!     ArgWithSrcList       = booleanParameter('withsrclist')
!     if (ArgWithSrcList) then
!       ArgSrcListSetTabName = stringParameter('srclisttab')
!     else
!       ArgWithCoords = booleanParameter('withcoords')
!     end if
! 
!     ArgWithSrcNum        = booleanParameter('withsrcnum')
!     if (ArgWithSrcNum .or. ArgWithSrcList) then
!       ! In the case of the latter, reads just the default value (=0).
!       ArgSrcNum            = intParameter('srcnum')
!       ! ArgSrcNum            = intParameter('srcidnumber')
!     else
!       ArgSrcNum            = -999
!     end if
! 
!     ! ArgSelectionExpression = stringParameter('expression')
! !    zClipFractionLo = realParameter('zlo')
! !    zClipFractionHi = realParameter('zhi')
! !    withGifNameRoot = booleanParameter('withgifroot')
! 
!     ArgWithCatSet = booleanParameter('withcat')
!     if (ArgWithCatSet) then
!       ArgCatSetTabName = stringParameter('cattab')
!     end if
! 
!     if (.not. ArgWithSrcList) then	! Single user-specified source
!       if (ArgWithCatSet) then
!         if (.not. ArgWithSrcNum) then
!           call error('NoSrcNumSpecified' &
!              , 'srcnum is mandatory when withsrclist=no and withcat=yes.')
!         end if
!       else if (.not. ArgWithCoords) then
!         call error('NoCoordsSpecified' &
!            , 'ra and dec are mandatory when withsrclist=no and withcat=no.')
!       end if
! 
!       if (ArgWithCoords) then
!         ! RA and DEC can be read, irrelevant to the value of withcat.
!         ArgRa  = realParameter('ra')
!         ArgDec = realParameter('dec')
!       end if
!     end if
! 
!     ArgIsAutoFname = booleanParameter('autofname')
!     if (ArgIsAutoFname) then
!       ArgGifNameRoot       = stringParameter('gifroot')
!       ArgFnameStyle        = stringParameter('fnamestyle')
!       select case(trim(ArgFnameStyle))
!       case('srcnum')
!         filenameStyle%srcNum = .true.
!         ! filenameWithSrcNum = .true.	! was defined in makethumbs_aux
!       case('hexsrcnum')
!         filenameStyle%srcNumHex = .true.
!         ! filenameWithSrcNumHex = .true.
!       case('detid')
!         filenameStyle%detId  = .true.
!         ! filenameWithDetId  = .true.
!       case('srcid')
!         filenameStyle%srcId  = .true.
!         ! filenameWithSrcId  = .true.
!       case('srcnum_detid')
!         filenameStyle%srcNum = .true.
!         filenameStyle%detId  = .true.
!         ! filenameWithSrcNum = .true.
!         ! filenameWithDetId  = .true.
!       case('srcnum_srcid')
!         filenameStyle%srcNum = .true.
!         filenameStyle%srcId  = .true.
!         ! filenameWithSrcNum = .true.
!         ! filenameWithSrcId  = .true.
!       case('all')
!         filenameStyle%srcNum = .true.
!         filenameStyle%srcNumHex = .true.
!         filenameStyle%detId  = .true.
!         filenameStyle%srcId  = .true.
!         ! filenameWithSrcNum = .true.
!         ! filenameWithSrcNumHex = .true.
!         ! filenameWithDetId  = .true.
!         ! filenameWithSrcId  = .true.
!       case default
!         call error('badArgument', 'Inappropriate fnamestyle option is given.')
!       end select
! 
!       ! filenameWithSrcNum    = booleanParameter('fnamewithsrcnum')
!       ! filenameWithSrcNumHex = booleanParameter('fnamewithsrcnumhex')
!       ! filenameWithDetId     = booleanParameter('fnamewithdetid')
!       ! if ((.not. filenameWithSrcNum) .and. (.not. filenameWithDetId) &
!       !     .and. (.not. filenameWithSrcNumHex)) then
!       !   call error('badArgument', &
!       !        'Either fnamewithsrcnum or fnamewithdetid (or both) has to be yes.')
!       ! end if
!       ArgFnameSeparator    = stringParameter('fnameseparator')
!       ArgGifSuffix         = stringParameter('gifsuffix')
! 
!     else
!       ArgOutGifName = stringParameter('outfilename')
!     end if	! if (ArgIsAutoFname) then
! 
!     ArgSrcIndexSeparator = stringParameter('srcindexseparator')
!     if (trim(ArgSrcIndexSeparator) == 'DEFAULT') then
!       isSrcIndexSeparatorDef = .true.
!     else
!       isSrcIndexSeparatorDef = .false.
!     end if
! 
!     ! printDetId  = booleanParameter('printdetid')
!     ArgSrcIndexStyle     = stringParameter('srcindexstyle')
! 
!     if (trim(ArgSrcIndexStyle) == 'DEFAULT') then
!       if (ArgWithCatSet) then
!         ArgSrcIndexStyle = 'detid'
!       else if (ArgWithSrcList) then
!         ArgSrcIndexStyle = 'srcnum'
!       else
!         ArgSrcIndexStyle = 'none'
!       end if
!     end if
! 
!     select case(trim(ArgSrcIndexStyle))
!     case('none')
!       tmpSeparator = ''
!     case('srcnum')
!       tmpSeparator = '/'
!     ! case('hexsrcnum')
!     !   call error('unsupportedArgument' &
!     !      , 'srcindexstyle=hexsrcnum is not yet supported.  Sorry!')
!     case('detid')
!       tmpSeparator = ':'
!     case('srcid')
!       tmpSeparator = '/'
!     case default
!       call error('badArgument', 'Inappropriate printindexstyle option is given.')
!     end select
! 
!     if (isSrcIndexSeparatorDef) then
!       ArgSrcIndexSeparator = trim(tmpSeparator)
!     end if
! 
!     ! select case(trim(ArgSrcIndexStyle))
!     ! case('none')
!     !   if (isSrcIndexSeparatorDef) ArgSrcIndexSeparator = ''
!     ! case('srcnum')
!     !   if (isSrcIndexSeparatorDef) ArgSrcIndexSeparator = '/'
!     ! ! case('hexsrcnum')
!     ! case('detid')
!     !   if (isSrcIndexSeparatorDef) ArgSrcIndexSeparator = ':'
!     ! case('srcid')
!     !   if (isSrcIndexSeparatorDef) ArgSrcIndexSeparator = '/'
!     ! case default
!     !   call error('badArgument', 'Inappropriate printindexstyle option is given.')
!     ! end select
! 
! !    imageTransferStyle = stringParameter('scaletype')
! 
! !    select case(imageTransferStyle)
! !    case('linear')
! !      imageTransferId = 0
! !    case('log')
! !      imageTransferId = 1
! !    case('sqrt')
! !      imageTransferId = 2
! !    case default
! !call error('badImageTransferId','')
! !    end select
! 
!     ArgIaunamePrefix     = stringParameter('iaunameprefix')
!     ArgObsIdStr    = stringParameter('obsidstr')
!     ArgERangeStr   = stringParameter('erangestr')
!     ArgInstStr     = stringParameter('inststr')
!     ArgCommLower   = stringParameter('commlower')
!     ArgCommUpper   = stringParameter('commupper')
! 
!     ArgImageSize      = realParameter('imagesize')
! 
!     ArgLWidth         = intParameter('lwidth')
!     ArgSizeIndexCross = realParameter('sizeratiocross')
!     ArgFontNumber     = intParameter('fontnumber')
!     ArgColourMapId    = intParameter('colourmapid')	! Default: 3 = heat
!     ! ArgColourMapId = intParameter('colourmap')
! 
!     ArgIsDryRun   = booleanParameter('dryrun')

  end subroutine readParameters


  !---------------------------------------------------------------------
  function getThumbnailName(thumbOne, fnStyle) result(retFilename)
  !---------------------------------------------------------------------
  ! Determine the (output) filename, using global variables.
  ! NOTE: Order = SRCNUM_SRCNUMHEX_DETID_SRCID

    implicit none 

    integer, parameter :: srcNumLength = 3	! 3 digits for SRC_NUM

    !character(len=maxFilenameOfThumbnail) :: getThumbnailName
    character(LENMAXTHUMBNAILFILENAME) :: retFilename
    type(ThumbOneT), intent(in) :: thumbOne	! Defined in makethumb_aux
    ! integer(int32), intent(in) :: srcNum, detId
    type(FilenameStyleT), intent(in) :: fnStyle	! This was set based on the command-line arguments

    character(len=srcNumLength) :: hexSrcNum
    character(len=1) :: strSrcNumLength
    character(10) :: strTmp

    !-------------------------------------------------------------------
    ! Start:

    retFilename = trim(ArgTask%gifNameRoot)

    if (fnStyle%srcNum) then
      retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(1:1))//trim(intToStr(thumbOne%srcIndex))
    end if

    if (fnStyle%srcNumHex) then
      write(strSrcNumLength,'(I1.1)') srcNumLength
      write(hexSrcNum,'(Z'//strSrcNumLength//'.'//strSrcNumLength//')') thumbOne%srcIndex
      retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(2:2))//hexSrcNum
    end if

    if (fnStyle%detId) then
      strTmp = intToStr(thumbOne%detid)
      retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(3:3))//trim(strTmp)
    end if

    if (fnStyle%srcId) then
      strTmp = intToStr(thumbOne%srcid)
      retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(4:4))//trim(strTmp)
    end if

    retFilename = trim(retFilename)//trim(ArgTask%gifSuffix)
    
  end function getThumbnailName


!   !---------------------------------------------------------------------
!   function getThumbnailName(srcNum, detId, fnStyle) result(retFilename)
!   !---------------------------------------------------------------------
!   ! Determine the (output) filename, using global variables.
! 
!     implicit none 
! 
!     integer, parameter :: srcNumLength = 3	! 3 digits for SRC_NUM
! 
!     !character(len=maxFilenameOfThumbnail) :: getThumbnailName
!     character(LENMAXTHUMBNAILFILENAME) :: retFilename
!     integer(int32), intent(in) :: srcNum, detId
!     type(FilenameStyleT), intent(in) :: fnStyle	! This was set based on the command-line arguments
! 
!     character(len=srcNumLength) :: hexSrcNum
!     character(len=1) :: strSrcNumLength
! 
!     !-------------------------------------------------------------------
!     ! Start:
! 
!     retFilename = trim(ArgTask%gifNameRoot)
! 
!     if (fnStyle%srcNum) then
!       retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(1:1))//trim(intToStr(srcNum))
!     end if
! 
!     if (fnStyle%srcNumHex) then
!       write(strSrcNumLength,'(I1.1)') srcNumLength
!       write(hexSrcNum,'(Z'//strSrcNumLength//'.'//strSrcNumLength//')') srcNum
!       retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(2:2))//hexSrcNum
!     end if
! 
!     if (fnStyle%detId) then
!       retFilename = trim(retFilename)//trim(ArgTask%fnameSeparator(3:3))//trim(intToStr(detId))
!     end if
! 
!     retFilename = trim(retFilename)//trim(ArgTask%gifSuffix)
!     
!   end function getThumbnailName


  !---------------------------------------------------------------------
  function isFileExistingAndQuit(filename)
  !---------------------------------------------------------------------
  ! True if carrying on (with a message printed possibly).

    implicit none 

    character(len=*), intent(in) :: filename
    logical :: isOutFileExisting, isFileExistingAndQuit

    isFileExistingAndQuit = .false.
    inquire(file=trim(filename),exist=isOutFileExisting)
    if (isOutFileExisting) then
      if (clobber()) then
        call message(AppMsg, Verbose&
             , 'File ('//trim(filename)//') exists.  Overwritten.')
      else
        call warning('fileExists'&
             , 'File ('//trim(filename)//') exists.  NOT overwritten.  Skipped.')
        isFileExistingAndQuit = .true.
      end if
    end if

  end function isFileExistingAndQuit



end module makethumbs_mod
