! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testingestsrcnamesaux
  ! Authors:  Masaaki Sakano - SSC/LUX
  ! NOTE: Creates the file outfiles/testinput_for_getstrs.fits in run.

  use errorhandling
  use types
  use dal		! 

  use ingestsrcnames_aux

  !! Followings are in ssclib
  !use constants		! for Pi
  !use test_utils	! for isNearlyEqual()

  implicit none

  character(36), parameter :: &
       OUTFILNAME  = "outfiles/testinput_for_getstrs.fits" &
     , OUTFILNAME2 = "outfiles/testinput_for_retkwd2.fits" &
     , OUTFILNAME3 = "outfiles/testinput_for_retkwd3.fits"
  character(80), parameter :: &
       INCATFILENAME = "testdata/testcat.fits"
  character(10), parameter :: TMPTBLNAME = "test_table"

  !-------------------------------------------------------------------
  ! Start:

  call create_for_getStrs
  call test_getObsIdInt32FromHeader
  call test_getStrs

  call create_for_retKwd
  call test_retKwdFromAnyBlockInt32
  call test_retKwdFromAnyBlockInt16
  call test_retKwdFromAnyBlockInt8
  call test_retKwdFromAnyBlockDouble
  call test_retKwdFromAnyBlockSingle
  call test_retKwdFromAnyBlockChar

  call test_getCat5Cols

contains

  !---------------------------------------------------------------------
  subroutine create_for_getStrs
    !---------------------------------------------------------------------
    ! Create a file for test_getStrs().
    ! Note: See /dal/tests/f90test/addattribute.f90 columnnumber.f90 setcolumn.f90

    implicit none

    type(DataSetT) :: outSet
    type(TableT)   :: outTab
    type(ColumnT)  :: outCol

    !-------------------------------------------------------------------
    ! Start:

    outSet = dataSet(OUTFILNAME, CREATE)
    call setAttribute(outSet,"OBS_ID","0123456789","Test string for OBS_ID")
    call setAttribute(outSet,"INSTRUME",'EPIC    ')

    call addComment(outSet &
         ,"this is just another comment that is to be added to the COMMENTS section of the output data set")

    outTab = addTable(outSet, TMPTBLNAME, 7)
    call setAttribute(outTab,"OBS_ID","0234567891","Test string for OBS_ID")
    outCol = addColumn(outTab, "SRC_NUM", INTEGER32)
    outCol = addColumn(outTab, "DETID",   INTEGER32)
    outCol = addColumn(outTab, "SRCID",   INTEGER32)
    outCol = addColumn(outTab, "INCATFLG", BOOLEAN)
    outCol = addColumn(outTab, "RA",      INTEGER32)
    outCol = addColumn(outTab, "DEC",     INTEGER32)
    outCol = addColumn(outTab, "DEC_UNC", INTEGER32)
    outCol = addColumn(outTab, "DEC_CORR",INTEGER32)
    outCol = addColumn(outTab, "IAUNAME", STRING, comment="char column", dimensions=(/LENGTHOFIAUNAME/))
    outCol = addColumn(outTab, "OBS_ID",  STRING, comment="Obs_Id col", dimensions=(/LENOBSIDSTR/))

    call release(outTab)
    call release(outSet)

  end subroutine create_for_getStrs

  !---------------------------------------------------------------------
  subroutine create_for_retKwd
  !---------------------------------------------------------------------
  ! Create a file for test_retKwdFromAnyBlock().
  ! Note: See /dal/tests/f90test/addattribute.f90 columnnumber.f90 setcolumn.f90

    implicit none

    type(DataSetT) :: outSet
    type(TableT)   :: outTab
    type(ColumnT)  :: outCol
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'

    !-------------------------------------------------------------------
    ! Start:

    outSet = dataSet(OUTFILNAME2, CREATE)
    call setAttribute(outSet, kwdInt,0)		! Primary
    call setAttribute(outSet, kwdChar,'EPIC    ')	! Primary

    outTab = addTable(outSet, TMPTBLNAME//"1", 1)
    outCol = addColumn(outTab, "SRC_NUM", INTEGER32)
    call release(outTab)

    outTab = addTable(outSet, TMPTBLNAME//"2", 1)
    outCol = addColumn(outTab, "DETID",   INTEGER32)
    call release(outTab)

    outTab = addTable(outSet, TMPTBLNAME//"3", 1)
    outCol = addColumn(outTab, "SRCID",   INTEGER32)
    call release(outTab)
    call release(outSet)


    outSet = dataSet(OUTFILNAME3, CREATE)

    outTab = addTable(outSet, TMPTBLNAME//"1", 1)
    call setAttribute(outTab, kwdInt,-1)		! Header+1
    call setAttribute(outTab, kwdChar,'EPN     ')	! Header+1
    outCol = addColumn(outTab, "SRC_NUM", INTEGER32)
    call release(outTab)

    outTab = addTable(outSet, TMPTBLNAME//"2", 1)
    outCol = addColumn(outTab, "DETID",   INTEGER32)
    call release(outTab)

    outTab = addTable(outSet, TMPTBLNAME//"3", 1)
    call setAttribute(outTab, kwdInt,-3)		! Header+3
    call setAttribute(outTab, kwdChar,'MOS2    ')	! Header+3
    outCol = addColumn(outTab, "SRCID",   INTEGER32)
    call release(outTab)
    call release(outSet)

  end subroutine create_for_retKwd



  !---------------------------------------------------------------------
  subroutine test_getObsIdInt32FromHeader
    !---------------------------------------------------------------------
    ! Tests of getObsIdInt32FromHeader()
    ! NOTE: 

    implicit none

    character(28)  :: errHead
    type(DataSetT) :: inSet
    type(TableT)   :: inTab
    integer(int32) :: obsId

    !-------------------------------------------------------------------
    ! Start:

    inSet = dataSet(OUTFILNAME, READ)
    inTab = table(inSet, 0)


    !----------------------------------------------------------
    ! getObsIdInt32FromHeader()
    !----------------------------------------------------------
    errHead = "GetObsIdInt32FromHeader001a"
    print *,"=== "//trim(errHead)//": FromSet"

    obsId = getObsIdInt32FromHeader(inSet)
    if (obsId .ne. 123456789_int32) then
       print *,"ObsId from Set is (",obsId,")"
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetObsIdInt32FromHeader002a"
    print *,"=== "//trim(errHead)//": FromTab"

    obsId = getObsIdInt32FromHeader(inTab)
    if (obsId .ne. 234567891_int32) then
       print *,"ObsId from Tab is (",obsId,")"
       call error("err"//trim(errHead), '')
    end if


    !-------------------------------------------------------------------
    ! Cleaning up

    call release(inTab)
    call release(inSet)

  end subroutine test_getObsIdInt32FromHeader


  !---------------------------------------------------------------------
  subroutine test_getStrs
    !---------------------------------------------------------------------
    ! Tests of various functions to get Strings
    ! NOTE: 

    implicit none

    character(28)  :: errHead
    type(DataSetT) :: inSet
    type(TableT)   :: inTab
    ! integer(int32) :: obsId
    character(LEN=LENSTRCOLNAME) :: colNameStr
    character(LEN=LENOBSIDSTR)   :: obsIdStr
    character(256) :: str

    !-------------------------------------------------------------------
    ! Start:

    inSet = dataSet(OUTFILNAME, READ)
    inTab = table(inSet, TMPTBLNAME)


    !----------------------------------------------------------
    ! getSrcNumStr()
    !----------------------------------------------------------
    errHead = "GetSrcNumStr001a"
    print *,"=== "//trim(errHead)//": Normal"

    colNameStr = getSrcNumStr(inTab)
    str = trim(colNameStr)
    if (trim(str) .ne. "SRC_NUM") then
       print *,"Column name for SRC_NUM is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetSrcNumStr002a"
    print *,"=== "//trim(errHead)//": With a single Keyword"

    colNameStr = getSrcNumStr(inTab, "SRC_NUM")	! Practically testing getColKwdStrFromKey() 
    str = trim(colNameStr)
    if (trim(str) .ne. "SRC_NUM") then
       print *,"Column name for SRC_NUM is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetSrcNumStr003a"
    print *,"=== "//trim(errHead)//": Normal (with noErr)"

    colNameStr = getSrcNumStr(inTab, noErr=.true.)	! Practically testing getColKwdStrFromKey() 
    str = trim(colNameStr)
    if (trim(str) .ne. "SRC_NUM") then
       print *,"Column name for SRC_NUM is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetSrcNumStr004a"
    print *,"=== "//trim(errHead)//": Abnormal (with noErr)"

    colNameStr = getSrcNumStr(inTab, "NoExist", .true.)	! Practically testing getColKwdStrFromKey() 
    str = trim(colNameStr)
    if (trim(str) .ne. "") then
       print *,"Returned values is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getColKwdStrFromAry()
    !----------------------------------------------------------
    errHead = "GetColKwdStr001a"
    print *,"=== "//errHead//": FromAry"

    colNameStr = getColKwdStr(inTab, (/ "NoExist", "SRC_NUM" /))
    str = trim(colNameStr)
    if (trim(str) .ne. "SRC_NUM") then
       print *,"Column name for SRC_NUM is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getObsIdStr()
    !----------------------------------------------------------
    errHead = "GetObsIdStr001a"
    print *,"=== "//errHead//": "

    obsIdStr = getObsIdStr(inTab)
    if (trim(obsIdStr) .ne. "OBS_ID") then
       print *,"Column name for OBS_ID is (",trim(obsIdStr),")"
       call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    ! getIAUNameStr()
    !----------------------------------------------------------
    errHead = "GetIAUNameStr001a"
    print *,"=== "//trim(errHead)//": Normal"

    colNameStr = getIAUNameStr(inTab)
    str = trim(colNameStr)
    if (trim(str) .ne. "IAUNAME") then
       print *,"Column name for IAUNAME is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getDetIdStr()
    !----------------------------------------------------------
    errHead = "GetDetIdStr001a"
    print *,"=== "//trim(errHead)//": Normal"

    colNameStr = getDetIdStr(inTab)
    str = trim(colNameStr)
    if (trim(str) .ne. "DETID") then
       print *,"Column name for DETID is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getSrcIdStr()
    !----------------------------------------------------------
    errHead = "GetSrcIdStr001a"
    print *,"=== "//trim(errHead)//": Normal"

    colNameStr = getSrcIdStr(inTab)
    str = trim(colNameStr)
    if (trim(str) .ne. "SRCID") then
       print *,"Column name for SRCID is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getInCatFlgStr()
    !----------------------------------------------------------
    errHead = "GetInCatFlgStr001a"
    print *,"=== "//trim(errHead)//": Normal"

    colNameStr = getInCatFlgStr(inTab)
    str = trim(colNameStr)
    if (trim(str) .ne. "INCATFLG") then
       print *,"Column name for INCATFLG is (",trim(str),")"
       call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! getRaStr()
    !----------------------------------------------------------
    errHead = "GetRaStr001a"
    print *,"=== "//errHead//": Normal"

    obsIdStr = getRaStr(inTab)
    str = trim(obsIdStr)
    if (trim(str) .ne. "RA") then
       print *,"Column name for RA is (",trim(str),")"
       call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetRaStr002a"
    print *,"=== "//errHead//": With kwd"

    obsIdStr = getRaStr(inTab, "RA")
    str = trim(obsIdStr)
    if (trim(str) .ne. "RA") then
       print *,"Column name for RA is (",trim(str),")"
       call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetRaStr003a"
    print *,"=== "//errHead//": isCorrPreferred=.true."

    obsIdStr = getRaStr(inTab, isCorrPreferred=.true.)
    str = trim(obsIdStr)
    if (trim(str) .ne. "RA") then	! Anyhow there is no column named RA_CORR
       print *,"Column name for RA is (",trim(str),")"
       call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    ! getDecStr()
    !----------------------------------------------------------
    errHead = "GetDecStr001a"
    print *,"=== "//errHead//": Normal"

    obsIdStr = getDecStr(inTab)
    str = trim(obsIdStr)
    if (trim(str) .ne. "DEC") then
       print *,"Column name for DEC is (",trim(str),")"
       call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetDecStr002a"
    print *,"=== "//errHead//": With kwd"

    obsIdStr = getDecStr(inTab, "DEC_UNC")
    str = trim(obsIdStr)
    if (trim(str) .ne. "DEC_UNC") then
       print *,"Column name for DEC_UNC is (",trim(str),")"
       call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetDecStr003a"
    print *,"=== "//errHead//": isCorrPreferred=.true."

    obsIdStr = getDecStr(inTab, isCorrPreferred=.true.)
    str = trim(obsIdStr)
    if (trim(str) .ne. "DEC_CORR") then
       print *,"Column name for DEC_CORR is (",trim(str),")"
       call error("err"//errHead, '')
    end if


    !-------------------------------------------------------------------
    ! Cleaning up

    call release(inTab)
    call release(inSet)

  end subroutine test_getStrs


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockInt32
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: 

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    integer(int32) :: outValue, numBlock
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    integer(int32), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt32FromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, numBlock=numBlock)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, numBlock=numBlock)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt32FromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt32FromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockInt32


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockInt16
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: Pretty much the copy of test_retKwdFromAnyBlockInt32()

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    integer(int16) :: outValue
    integer(int32) :: numBlock
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    integer(int16), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt16FromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, numBlock=numBlock)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, numBlock=numBlock)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt16FromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt16FromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockInt16


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockInt8
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: Pretty much the copy of test_retKwdFromAnyBlockInt32()

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    integer(int8) :: outValue
    integer(int32) :: numBlock
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    integer(int8), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt8FromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, numBlock=numBlock)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, numBlock=numBlock)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockInt8FromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockInt8FromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockInt8


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockDouble
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: Pretty much the copy of test_retKwdFromAnyBlockInt32()

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    real(double) :: outValue
    integer(int32) :: numBlock
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    real(double), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockDoubleFromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, numBlock=numBlock)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, numBlock=numBlock)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockDoubleFromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockDoubleFromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockDouble


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockSingle
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: Pretty much the copy of test_retKwdFromAnyBlockInt32()

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    real(single) :: outValue
    integer(int32) :: numBlock
    character(8), parameter :: kwdInt = 'INT_TEST', kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    real(single), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockSingleFromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, numBlock=numBlock)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, numBlock=numBlock)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
       print *,"Column (",trim(kwdInt),") is ",outValue
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockSingleFromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. 0) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (aryValues(-1) .ne. 0) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = -9
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdInt &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (outValue .ne. -1) then	! See create_for_retKwd()
      print *,"Column (",trim(kwdInt),") is ",outValue
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdInt),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdInt),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdInt),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdInt),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((aryValues(0) .ne. -1) .or. (aryValues(2) .ne. -3)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdInt),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockSingleFromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdInt),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdInt),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdInt),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockSingle


  !---------------------------------------------------------------------
  subroutine test_retKwdFromAnyBlockChar
  !---------------------------------------------------------------------
  ! Tests of various functions to get header values.
  ! NOTE: Pretty much the copy of test_retKwdFromAnyBlockInt32()

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: inSet2, inSet3
    character(32) :: outValue
    integer(int32) :: numBlock
    character(8), parameter :: kwdChar = 'INSTRUME'
    logical :: isFound, isErr
    character(32), pointer :: aryValues(:)
    logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    inSet2 = dataSet(OUTFILNAME2, READ)
    inSet3 = dataSet(OUTFILNAME3, READ)

    !----------------------------------------------------------
    ! retKwdFromAnyBlockCharFromSet()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet001a"
    print *,"=== "//trim(errHead)//": Simplest"

    call retKwdFromAnyBlock(outValue, inSet2, kwdChar)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet002a"
    print *,"=== "//trim(errHead)//": with numBlock"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdChar, numBlock=numBlock)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet003a"
    print *,"=== "//trim(errHead)//": with isFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdChar, isFound=isFound)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet004a"
    print *,"=== "//trim(errHead)//": with numBlock & isFound"

    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet2, kwdChar &
       , numBlock=numBlock, isFound=isFound)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet005a"
    print *,"=== "//trim(errHead)//": with aryValues & aryFound"

    call retKwdFromAnyBlock(outValue, inSet2, kwdChar &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet005b"
    print *,"=== "//trim(errHead)//": with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (trim(aryValues(-1)) .ne. 'EPIC') then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet005c"
    print *,"=== "//trim(errHead)//": with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, inSet2, kwdChar &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (trim(aryValues(-1)) .ne. 'EPIC') then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet011a"
    print *,"=== "//trim(errHead)//": File3: Simplest"

    outValue = ''
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet012a"
    print *,"=== "//trim(errHead)//": File3: with numBlock"

    outValue = ''
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar, numBlock=numBlock)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet013a"
    print *,"=== "//trim(errHead)//": File3: with isFound"

    outValue = ''
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar, isFound=isFound)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet014a"
    print *,"=== "//trim(errHead)//": File3: with numBlock & isFound"

    outValue = ''
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar &
       , numBlock=numBlock, isFound=isFound)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
       print *,"Column (",trim(kwdChar),") is ",trim(outValue)
       call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet015a"
    print *,"=== "//trim(errHead)//": File3: with aryValues & aryFound"

    outValue = ''
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet015b"
    print *,"=== "//trim(errHead)//": File3: with <aryValues> & aryFound"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((trim(aryValues(0)) .ne. 'EPN') &
        .or. (trim(aryValues(2)) .ne. 'MOS2')) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet015c"
    print *,"=== "//trim(errHead)//": File3: with aryValues & <aryFound>"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = ''
    numBlock = -9
    call retKwdFromAnyBlock(outValue, inSet3, kwdChar &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((trim(aryValues(0)) .ne. 'EPN') &
        .or. (trim(aryValues(2)) .ne. 'MOS2')) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    call release(inSet3)	!!! Never forget!!
    call release(inSet2)	!!! Never forget!!
    !----------------------------------------------------------


    !----------------------------------------------------------
    ! retKwdFromAnyBlockCharFromName()
    !----------------------------------------------------------
    !! With the datafile 2
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromName006a"
    print *,"=== "//trim(errHead)//": with all (test numBlock, isFound)"

    call retKwdFromAnyBlock(outValue, OUTFILNAME2, kwdChar &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPIC') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. -1) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromName006b"
    print *,"=== "//trim(errHead)//": with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if (trim(aryValues(-1)) .ne. 'EPIC') then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromSet006c"
    print *,"=== "//trim(errHead)//": with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if ((.not. aryFound(-1)) .or. aryFound(0)) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!


    !----------------------------------------------------------
    !! With the datafile 3
    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromName016a"
    print *,"=== "//trim(errHead)//": File3: with all (test numBlock, isFound)"

    outValue = ''
    numBlock = -9
    call retKwdFromAnyBlock(outValue, OUTFILNAME3, kwdChar &
       , numBlock=numBlock, isFound=isFound &
       , aryValues=aryValues, aryFound=aryFound)

    if (trim(outValue) .ne. 'EPN') then	! See create_for_retKwd()
      print *,"Column (",trim(kwdChar),") is ",trim(outValue)
      call error("err"//trim(errHead), '')
    else if (numBlock .ne. 0) then	! See create_for_retKwd()
       print *,"numBlock for Column (",trim(kwdChar),") is ",numBlock
       call error("err"//trim(errHead), '')
    else if (.not. isFound) then	! See create_for_retKwd()
       print *,"isFound for Column (",trim(kwdChar),") is ",isFound
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromName016b"
    print *,"=== "//trim(errHead)//": File3: with all (test aryValues)"

    isErr = .false.
    if (     lbound(aryValues,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryValues) for Column (",trim(kwdChar),") is ", lbound(aryValues,1)
      isErr = .true.
    else if (ubound(aryValues,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryValues) for Column (",trim(kwdChar),") is ", ubound(aryValues,1)
      isErr = .true.
    else if ((trim(aryValues(0)) .ne. 'EPN') &
        .or. (trim(aryValues(2)) .ne. 'MOS2')) then	! See create_for_retKwd()
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryValues for Column (",trim(kwdChar),") is ", aryValues
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetKwdFromAnyBlockCharFromName016c"
    print *,"=== "//trim(errHead)//": File3: with all (test aryFound)"

    isErr = .false.
    if (     lbound(aryFound,1) .ne. -1) then	! See create_for_retKwd()
      print *,"lbound(aryFound) for Column (",trim(kwdChar),") is ", lbound(aryFound,1)
      isErr = .true.
    else if (ubound(aryFound,1) .ne. 2) then	! See create_for_retKwd()
      print *,"ubound(aryFound) for Column (",trim(kwdChar),") is ", ubound(aryFound,1)
      isErr = .true.
    else if (.not. ( &
             (.not. aryFound(-1)) &
       .and. (      aryFound(0))  &
       .and. (.not. aryFound(1))  &
       .and. (      aryFound(2)) )) then
      isErr = .true.
    end if
    if (isErr) then
      print *,"aryFound for Column (",trim(kwdChar),") is ", aryFound
      call error("err"//trim(errHead), '')
    end if

    deallocate(aryValues, aryFound)	!!! Never forget!!

  end subroutine test_retKwdFromAnyBlockChar


  !---------------------------------------------------------------------
  subroutine test_getCat5Cols
  !---------------------------------------------------------------------
  ! Tests of subroutines to get columns from a external catalogue.
  ! NOTE: 

    implicit none

    character(64)  :: errHead
    type(DataSetT) :: catSet
    type(TableT)   :: catTab
    type(SrcPrmT)  :: catPrm
    integer(int32) :: numRows

    type valIntSrcPrm
      integer(int32) :: obsId, srcNum, detid, srcid &
         , inCatFlag, catName, raDeg, decDeg
      ! type(ColumnT)   :: catNameCol
    end type valIntSrcPrm

    type(valIntSrcPrm) :: lboundCatPrm, sizeCatPrm

    ! character(32) :: outValue
    ! integer(int32) :: numBlock
    ! character(8), parameter :: kwdChar = 'INSTRUME'
    ! logical :: isFound, isErr
    ! character(32), pointer :: aryValues(:)
    ! logical,        pointer :: aryFound(:)

    !-------------------------------------------------------------------
    ! Start:

    catSet = dataSet(INCATFILENAME, READ)
    catTab = table(catSet, 0)
    numRows = numberOfRows(catTab)

    call getCat5Cols(catTab, catPrm, getRaDec=.false., getCatName=.true.)	! nb., getCatName=.true. is NOT default.

    call release(catTab)	! catPrm%catNameCol is now released.
    call release(catSet)

    !----------------------------------------------------------
    ! getCatPrmFromTab()
    !----------------------------------------------------------
    errHead = "GetCatPrmFromTab001a"
    print *,"=== "//trim(errHead)//": Row-starting number"

    lboundCatPrm%obsId   = lbound(catPrm%obsId,   1)
    lboundCatPrm%srcNum  = lbound(catPrm%srcNum,  1)
    lboundCatPrm%detid   = lbound(catPrm%detid,   1)
    lboundCatPrm%srcid   = lbound(catPrm%srcid,   1)
    lboundCatPrm%catName = lbound(catPrm%catName, 1)

    sizeCatPrm%obsId     =   size(catPrm%obsId,   1)
    sizeCatPrm%srcNum    =   size(catPrm%srcNum,  1)
    sizeCatPrm%detid     =   size(catPrm%detid,   1)
    sizeCatPrm%srcid     =   size(catPrm%srcid,   1)
    sizeCatPrm%catName   =   size(catPrm%catName, 1)

    if ((lboundCatPrm%obsId .ne. lboundCatPrm%srcNum) .or. &
        (lboundCatPrm%obsId .ne. lboundCatPrm%detid)  .or. &
        (lboundCatPrm%obsId .ne. lboundCatPrm%srcid)  .or. &
        (lboundCatPrm%obsId .ne. lboundCatPrm%catName) ) then
       print *,"lbound of catPrm%obsId  is ",lboundCatPrm%obsId 
       print *,"lbound of catPrm%srcNum is ",lboundCatPrm%srcNum
       print *,"lbound of catPrm%detid  is ",lboundCatPrm%detid 
       print *,"lbound of catPrm%srcid  is ",lboundCatPrm%srcid 
       print *,"lbound of catPrm%catName is ",lboundCatPrm%catName
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetCatPrmFromTab002a"
    print *,"=== "//trim(errHead)//": Size of rows (catPrm)"

    if ((sizeCatPrm%obsId .ne. sizeCatPrm%srcNum) .or. &
        (sizeCatPrm%obsId .ne. sizeCatPrm%detid)  .or. &
        (sizeCatPrm%obsId .ne. sizeCatPrm%srcid)  .or. &
        (sizeCatPrm%obsId .ne. sizeCatPrm%catName) ) then
       print *,"size of catPrm%obsId  is ",sizeCatPrm%obsId 
       print *,"size of catPrm%srcNum is ",sizeCatPrm%srcNum
       print *,"size of catPrm%detid  is ",sizeCatPrm%detid 
       print *,"size of catPrm%srcid  is ",sizeCatPrm%srcid 
       print *,"size of catPrm%catName is ",sizeCatPrm%catName
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetCatPrmFromTab003a"
    print *,"=== "//trim(errHead)//": Size of rows (catPrm%obsId:",numRows,"?)"

    if (sizeCatPrm%obsId .ne. numRows) then
       print *,"size of catPrm%obsId  is ",sizeCatPrm%obsId 
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "GetCatPrmFromTab004a"
    print *,"=== "//trim(errHead)//": Check values (catPrm%*)"
    !     iauname               detid       srcid       src_num     obs_id
    ! 
    !   1 2XMM J054607.9-691002       69872       53956           3  094411101

    if ((catPrm%obsId( lboundCatPrm%obsId)  .ne. 94411101_int32) .or. &
        (catPrm%srcNum(lboundCatPrm%srcNum) .ne. 3) .or. &
        (catPrm%detid( lboundCatPrm%detid)  .ne. 69872_int32) .or. &
        (catPrm%srcid( lboundCatPrm%srcid)  .ne. 53956_int32) .or. &
        (trim(catPrm%catName(lboundCatPrm%catName))  .ne. '2XMM J054607.9-691002') ) then
       print *,"catPrm%obsId(1st)   is not 94411101 but",catPrm%obsId(lboundCatPrm%obsId)
       print *,"catPrm%srcNum(1st)  is not 3 but",      catPrm%srcNum(lboundCatPrm%srcNum)
       print *,"catPrm%detid(1st)   is not 69872 but",   catPrm%detid(lboundCatPrm%detid)
       print *,"catPrm%srcid(1st)   is not 53956 but",   catPrm%srcid(lboundCatPrm%srcid)
       print *,'catPrm%catName(1st) is not "2XMM J054607.9-691002" but (' &
          , trim(catPrm%catName(lboundCatPrm%catName)), ')'
       call error("err"//trim(errHead), '')
    end if

    call deallocateSrcPrm(catPrm)

  end subroutine test_getCat5Cols

end program testingestsrcnamesaux

