! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testmakethumbsaux
  ! Authors:  Masaaki Sakano - SSC/LUX
  ! NOTE: 
  !  Make sure to run ./testingestsrcnamesaux before this, for
  !  it creates the input data file this code reads.

  use errorhandling
  use types
  use dal		! 

  use makethumbs_aux

  !! Followings are in ssclib
  ! use cal_aux		! for 
  ! use test_utils	! for isNearlyEqual()

  implicit none

  character(35), parameter :: INFILNAME = "outfiles/testinput_for_getstrs.fits"
  ! character(10), parameter :: TMPTBLNAME = "test_table"

  !-------------------------------------------------------------------
  ! Start:

  call test_getInstStr
  call test_writenumber
  call test_retDirname
  call test_isDirectoryReadable
  call test_isDirnameReadable
  call test_isSrcWithinFOV

contains

  !---------------------------------------------------------------------
  subroutine test_getInstStr
  !---------------------------------------------------------------------
  ! Tests of getInstStr()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    type(DataSetT) :: inSet
    ! type(TableT)   :: inTab
    character(LENINSTSTRPRINT) :: instStr

    !-------------------------------------------------------------------
    ! Start:

    inSet = dataSet(INFILNAME, READ)
    ! inTab = table(inSet, 0)

    !----------------------------------------------------------
    ! getInstStr()
    !----------------------------------------------------------
    errHead = "GetInstStr001a"
    print *,"=== "//trim(errHead)//": From the test dataSet"

    instStr = getInstStr(inSet)
    if (trim(instStr) .ne. 'EP') then
      print *,"Instrument to print is (",trim(instStr),")"
      call error("err"//trim(errHead), '')
    end if

    !-------------------------------------------------------------------
    ! Cleaning up

    ! call release(inTab)
    call release(inSet)

  end subroutine test_getInstStr


  !---------------------------------------------------------------------
  subroutine test_writenumber
  !---------------------------------------------------------------------
  ! Tests of writenumber()
  ! NOTE: 

    implicit none

    character(32)  :: errHead
    character(64)  :: retStr, str

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! writeint8()
    !----------------------------------------------------------
    errHead = "writeint8_001a"
    print *,"=== "//trim(errHead)//": No specified format (Int8)."

    call writenumber(3_int8, var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint8_002a"
    print *,"=== "//trim(errHead)//": Specified format I1 (Int8)."

    call writenumber(3_int8, fmt='(I1)', var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint8_003a"
    print *,"=== "//trim(errHead)//": Specified format I4 (Int8)."

    call writenumber(3_int8, fmt='(I4)', var=retStr)

    if (trim(retStr) .ne. '   3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint8_004a"
    print *,"=== "//trim(errHead)//": Specified format I4.2 (Int8)."

    call writenumber(3_int8, fmt='(I4.2)', var=retStr)

    if (trim(retStr) .ne. '  03') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint8_005a"
    print *,"=== "//trim(errHead)//": No specified format (Int8, negative)."

    call writenumber(-3_int8, var=retStr)

    if (trim(retStr) .ne. '-3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! writeint16()
    !----------------------------------------------------------
    errHead = "writeint16_001a"
    print *,"=== "//trim(errHead)//": No specified format (Int16)."

    call writenumber(3_int16, var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint16_002a"
    print *,"=== "//trim(errHead)//": Specified format I1 (Int16)."

    call writenumber(3_int16, fmt='(I1)', var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! writeint32()
    !----------------------------------------------------------
    errHead = "writeint32_001a"
    print *,"=== "//trim(errHead)//": No specified format (Int32)."

    call writenumber(3_int32, var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writeint32_002a"
    print *,"=== "//trim(errHead)//": Specified format I1 (Int32)."

    call writenumber(3_int32, fmt='(I1)', var=retStr)

    if (trim(retStr) .ne. '3') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! writereal()
    !----------------------------------------------------------
    errHead = "writereal_001a"
    print *,"=== "//trim(errHead)//": No specified format (Real, positive)."
 
    call writenumber(3.0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '3.000') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writereal_002a"
    print *,"=== "//trim(errHead)//": Specified format F3.1 (Real, positive)."

    call writenumber(3.0, fmt='(F3.1)', var=retStr)

    if (trim(retStr) .ne. '3.0') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writereal_003a"
    print *,"=== "//trim(errHead)//": No specified format (Real, zero)."
 
    call writenumber(0.0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '0.000') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writereal_004a"
    print *,"=== "//trim(errHead)//": No specified format (Real, zero)."
 
    call writenumber(-0.0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '-0.00') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writereal_005a"
    print *,"=== "//trim(errHead)//": No specified format (Real, negative)."
 
    call writenumber(-3.0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '-3.00') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writereal_006a"
    print *,"=== "//trim(errHead)//": Specified format F5.1 (Real, negative)."

    call writenumber(-3.0, fmt='(F5.1)', var=retStr)

    if (trim(retStr) .ne. ' -3.0') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    ! writedouble()
    !----------------------------------------------------------
    errHead = "writedouble_001a"
    print *,"=== "//trim(errHead)//": No specified format (Double, positive)."
 
    call writenumber(3d0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '3.000') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writedouble_002a"
    print *,"=== "//trim(errHead)//": Specified format F3.1 (Double, positive)."

    call writenumber(3d0, fmt='(F3.1)', var=retStr)

    if (trim(retStr) .ne. '3.0') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writedouble_003a"
    print *,"=== "//trim(errHead)//": No specified format (Double, zero)."
 
    call writenumber(0d0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '0.000') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writedouble_004a"
    print *,"=== "//trim(errHead)//": No specified format (Double, zero)."
 
    call writenumber(-0d0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '-0.00') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writedouble_005a"
    print *,"=== "//trim(errHead)//": No specified format (Double, negative)."
 
    call writenumber(-3d0, var=retStr)
    str = retStr(1:5)

    if (trim(str) .ne. '-3.00') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "writedouble_006a"
    print *,"=== "//trim(errHead)//": Specified format F5.1 (Double, negative)."

    call writenumber(-3d0, fmt='(F5.1)', var=retStr)

    if (trim(retStr) .ne. ' -3.0') then
      print *,"Returned string is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_writenumber


  !---------------------------------------------------------------------
  subroutine test_retDirname
  !---------------------------------------------------------------------
  ! Tests of retDirname()
  ! NOTE: 

    implicit none

    character(32)  :: errHead
    character(4096) :: retStr

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! retDirname()
    !----------------------------------------------------------
    errHead = "RetDirname001a"
    print *,"=== "//trim(errHead)//": "

    call retDirname('/a/b/c', retStr)
    if (trim(retStr) .ne. '/a/b') then
      print *,"Returned directory is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetDirname002a"
    print *,"=== "//trim(errHead)//": "

    call retDirname('/a/b/c/', retStr)
    if (trim(retStr) .ne. '/a/b/c') then
      print *,"Returned directory is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "RetDirname003a"
    print *,"=== "//trim(errHead)//": "

    call retDirname('  ../a/b/c', retStr)
    if (trim(retStr) .ne. '  ../a/b') then
      print *,"Returned directory is (",trim(retStr),")"
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_retDirname


  !---------------------------------------------------------------------
  subroutine test_isDirectoryReadable
  !---------------------------------------------------------------------
  ! Tests of isDirectoryReadable()
  ! NOTE: 

    implicit none

    character(32)  :: errHead

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! isDirectoryReadable()
    !----------------------------------------------------------
    errHead = "IsDirectoryReadable001a"
    print *,"=== "//trim(errHead)//": /tmp"

    if (.not. isDirectoryReadable('/tmp')) then
      print *,"/tmp is judged as not readable..."
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "IsDirectoryReadable001a"
    print *,"=== "//trim(errHead)//": /sonnanonaiyo"

    if (isDirectoryReadable('/sonnanonaiyo')) then
      print *,"/sonnanonaiyo is judged as readable..."
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_isDirectoryReadable


  !---------------------------------------------------------------------
  subroutine test_isDirnameReadable
  !---------------------------------------------------------------------
  ! Tests of isDirnameReadable()
  ! NOTE: 

    implicit none

    character(32)  :: errHead

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! isDirnameReadable()
    !----------------------------------------------------------
    errHead = "IsDirnameReadable001a"
    print *,"=== "//trim(errHead)//": /tmp/new_file_xxx"

    if (.not. isDirnameReadable('/tmp/new_file_xxx')) then
      print *,"/tmp is judged as not readable..."
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "IsDirnameReadable002a"
    print *,"=== "//trim(errHead)//": /sonnanonaiyo/new_file_xxx"

    if (isDirnameReadable('/sonnanonaiyo/new_file_xxx')) then
      print *,"/sonnanonaiyo is judged as readable..."
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_isDirnameReadable


  !---------------------------------------------------------------------
  subroutine test_isSrcWithinFOV
  !---------------------------------------------------------------------
  ! Tests of isSrcWithinFOV()
  ! NOTE: 

    implicit none

    character(32)  :: errHead
    type(ThumbOneT) :: thumbThis
    type(ThumbCommonT) :: thumbCommon

    !-------------------------------------------------------------------
    ! Start:

    thumbCommon%tSize%orgImageXSize = 10
    thumbCommon%tSize%orgImageYSize = 20

    !----------------------------------------------------------
    ! isSrcWithinFOV()
    !----------------------------------------------------------
    errHead = "isSrcWithinFOV001a"
    print *,"=== "//trim(errHead)//": "

    thumbThis%xPixel = 5
    thumbThis%yPixel = 5

    if (.not. isSrcWithinFOV(thumbThis, thumbCommon)) then
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "isSrcWithinFOV002a"
    print *,"=== "//trim(errHead)//": "

    thumbThis%xPixel = 0
    thumbThis%yPixel = 5

    if (isSrcWithinFOV(thumbThis, thumbCommon)) then
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "isSrcWithinFOV003a"
    print *,"=== "//trim(errHead)//": "

    thumbThis%xPixel = 5
    thumbThis%yPixel = 0

    if (isSrcWithinFOV(thumbThis, thumbCommon)) then
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "isSrcWithinFOV004a"
    print *,"=== "//trim(errHead)//": "

    thumbThis%xPixel = 100
    thumbThis%yPixel = 5

    if (isSrcWithinFOV(thumbThis, thumbCommon)) then
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "isSrcWithinFOV005a"
    print *,"=== "//trim(errHead)//": "

    thumbThis%xPixel = 5
    thumbThis%yPixel = 100

    if (isSrcWithinFOV(thumbThis, thumbCommon)) then
      call error("err"//trim(errHead), '')
    end if


  end subroutine test_isSrcWithinFOV


end program testmakethumbsaux

